@@ -37,8 +37,8 @@
 #include "bswapdsp.h"
 #include "bytestream.h"
 #include "codec_internal.h"
+#include "decode.h"
 #include "get_bits.h"
-#include "internal.h"
 
 
 #define BLOCK_TYPE_VLC_BITS 5
@@ -257,10 +257,10 @@ static av_cold void init_vlcs(void)
         for (j = 0; j < 4; j++) {
             block_type_vlc[i][j].table           = table[i][j];
             block_type_vlc[i][j].table_allocated = 32;
-            init_vlc(&block_type_vlc[i][j], BLOCK_TYPE_VLC_BITS, 7,
+            vlc_init(&block_type_vlc[i][j], BLOCK_TYPE_VLC_BITS, 7,
                      &block_type_tab[i][j][0][1], 2, 1,
                      &block_type_tab[i][j][0][0], 2, 1,
-                     INIT_VLC_USE_NEW_STATIC);
+                     VLC_INIT_USE_STATIC);
         }
     }
 }
@@ -706,8 +706,8 @@ static const uint8_t *read_huffman_tables(FourXContext *f,
         len_tab[j]  = len;
     }
 
-    ff_free_vlc(&f->pre_vlc);
-    if (init_vlc(&f->pre_vlc, ACDC_VLC_BITS, 257, len_tab, 1, 1,
+    ff_vlc_free(&f->pre_vlc);
+    if (vlc_init(&f->pre_vlc, ACDC_VLC_BITS, 257, len_tab, 1, 1,
                  bits_tab, 4, 4, 0))
         return NULL;
 
@@ -875,7 +875,7 @@ static int decode_frame(AVCodecContext *avctx, AVFrame *picture,
         }
 
         for (i = 0; i < CFRAME_BUFFER_COUNT; i++)
-            if (f->cfrm[i].id && f->cfrm[i].id < avctx->frame_number)
+            if (f->cfrm[i].id && f->cfrm[i].id < avctx->frame_num)
                 av_log(f->avctx, AV_LOG_ERROR, "lost c frame %d\n",
                        f->cfrm[i].id);
 
@@ -912,9 +912,9 @@ static int decode_frame(AVCodecContext *avctx, AVFrame *picture,
             buf        = cfrm->data;
             frame_size = cfrm->size;
 
-            if (id != avctx->frame_number)
-                av_log(f->avctx, AV_LOG_ERROR, "cframe id mismatch %d %d\n",
-                       id, avctx->frame_number);
+            if (id != avctx->frame_num)
+                av_log(f->avctx, AV_LOG_ERROR, "cframe id mismatch %d %"PRId64"\n",
+                       id, avctx->frame_num);
 
             if (f->version <= 1)
                 return AVERROR_INVALIDDATA;
@@ -952,12 +952,17 @@ static int decode_frame(AVCodecContext *avctx, AVFrame *picture,
     } else if (frame_4cc == AV_RL32("snd_")) {
         av_log(avctx, AV_LOG_ERROR, "ignoring snd_ chunk length:%d\n",
                buf_size);
+        return AVERROR_INVALIDDATA;
     } else {
         av_log(avctx, AV_LOG_ERROR, "ignoring unknown chunk length:%d\n",
                buf_size);
+        return AVERROR_INVALIDDATA;
     }
 
-    picture->key_frame = picture->pict_type == AV_PICTURE_TYPE_I;
+    if (picture->pict_type == AV_PICTURE_TYPE_I)
+        picture->flags |= AV_FRAME_FLAG_KEY;
+    else
+        picture->flags &= ~AV_FRAME_FLAG_KEY;
 
     av_image_copy_plane(picture->data[0], picture->linesize[0],
                         (const uint8_t*)f->frame_buffer,  avctx->width * 2,
@@ -966,8 +971,6 @@ static int decode_frame(AVCodecContext *avctx, AVFrame *picture,
 
     *got_frame = 1;
 
-    emms_c();
-
     return buf_size;
 }
 
@@ -984,7 +987,7 @@ static av_cold int decode_end(AVCodecContext *avctx)
         av_freep(&f->cfrm[i].data);
         f->cfrm[i].allocated_size = 0;
     }
-    ff_free_vlc(&f->pre_vlc);
+    ff_vlc_free(&f->pre_vlc);
 
     return 0;
 }
@@ -1014,7 +1017,7 @@ static av_cold int decode_init(AVCodecContext *avctx)
         return AVERROR(ENOMEM);
 
     f->version = AV_RL32(avctx->extradata) >> 16;
-    ff_blockdsp_init(&f->bdsp, avctx);
+    ff_blockdsp_init(&f->bdsp);
     ff_bswapdsp_init(&f->bbdsp);
     f->avctx = avctx;
 
@@ -1030,7 +1033,7 @@ static av_cold int decode_init(AVCodecContext *avctx)
 
 const FFCodec ff_fourxm_decoder = {
     .p.name         = "4xm",
-    .p.long_name    = NULL_IF_CONFIG_SMALL("4X Movie"),
+    CODEC_LONG_NAME("4X Movie"),
     .p.type         = AVMEDIA_TYPE_VIDEO,
     .p.id           = AV_CODEC_ID_4XM,
     .priv_data_size = sizeof(FourXContext),
@@ -1038,5 +1041,5 @@ const FFCodec ff_fourxm_decoder = {
     .close          = decode_end,
     FF_CODEC_DECODE_CB(decode_frame),
     .p.capabilities = AV_CODEC_CAP_DR1,
-    .caps_internal  = FF_CODEC_CAP_INIT_THREADSAFE | FF_CODEC_CAP_INIT_CLEANUP,
+    .caps_internal  = FF_CODEC_CAP_INIT_CLEANUP,
 };
