@@ -35,16 +35,15 @@
 
 #include "config_components.h"
 
-#include <stdio.h>
-#include <stdlib.h>
+#include <stddef.h>
 #include <zlib.h>
 
 #include "libavutil/intreadwrite.h"
 #include "avcodec.h"
 #include "bytestream.h"
 #include "codec_internal.h"
+#include "decode.h"
 #include "get_bits.h"
-#include "internal.h"
 #include "zlib_wrapper.h"
 
 typedef struct BlockInfo {
@@ -503,7 +502,7 @@ static int flashsv_decode_frame(AVCodecContext *avctx, AVFrame *rframe,
 #if CONFIG_FLASHSV_DECODER
 const FFCodec ff_flashsv_decoder = {
     .p.name         = "flashsv",
-    .p.long_name    = NULL_IF_CONFIG_SMALL("Flash Screen Video v1"),
+    CODEC_LONG_NAME("Flash Screen Video v1"),
     .p.type         = AVMEDIA_TYPE_VIDEO,
     .p.id           = AV_CODEC_ID_FLASHSV,
     .priv_data_size = sizeof(FlashSVContext),
@@ -511,7 +510,7 @@ const FFCodec ff_flashsv_decoder = {
     .close          = flashsv_decode_end,
     FF_CODEC_DECODE_CB(flashsv_decode_frame),
     .p.capabilities = AV_CODEC_CAP_DR1,
-    .caps_internal  = FF_CODEC_CAP_INIT_THREADSAFE | FF_CODEC_CAP_INIT_CLEANUP,
+    .caps_internal  = FF_CODEC_CAP_INIT_CLEANUP,
     .p.pix_fmts     = (const enum AVPixelFormat[]) { AV_PIX_FMT_BGR24, AV_PIX_FMT_NONE },
 };
 #endif /* CONFIG_FLASHSV_DECODER */
@@ -571,7 +570,7 @@ static av_cold int flashsv2_decode_end(AVCodecContext *avctx)
 
 const FFCodec ff_flashsv2_decoder = {
     .p.name         = "flashsv2",
-    .p.long_name    = NULL_IF_CONFIG_SMALL("Flash Screen Video v2"),
+    CODEC_LONG_NAME("Flash Screen Video v2"),
     .p.type         = AVMEDIA_TYPE_VIDEO,
     .p.id           = AV_CODEC_ID_FLASHSV2,
     .priv_data_size = sizeof(FlashSVContext),
@@ -579,7 +578,7 @@ const FFCodec ff_flashsv2_decoder = {
     .close          = flashsv2_decode_end,
     FF_CODEC_DECODE_CB(flashsv_decode_frame),
     .p.capabilities = AV_CODEC_CAP_DR1,
-    .caps_internal  = FF_CODEC_CAP_INIT_THREADSAFE | FF_CODEC_CAP_INIT_CLEANUP,
+    .caps_internal  = FF_CODEC_CAP_INIT_CLEANUP,
     .p.pix_fmts     = (const enum AVPixelFormat[]) { AV_PIX_FMT_BGR24, AV_PIX_FMT_NONE },
 };
 #endif /* CONFIG_FLASHSV2_DECODER */
