@@ -26,9 +26,11 @@
 #include "cbs.h"
 #include "cbs_bsf.h"
 #include "cbs_h264.h"
+#include "cbs_sei.h"
 #include "h264.h"
 #include "h264_levels.h"
-#include "h264_sei.h"
+#include "h2645data.h"
+#include "sei.h"
 
 enum {
     FLIP_HORIZONTAL = 1,
@@ -144,25 +146,17 @@ static int h264_metadata_update_sps(AVBSFContext *bsf,
     int crop_unit_x, crop_unit_y;
 
     if (ctx->sample_aspect_ratio.num && ctx->sample_aspect_ratio.den) {
-        // Table E-1.
-        static const AVRational sar_idc[] = {
-            {   0,  0 }, // Unspecified (never written here).
-            {   1,  1 }, {  12, 11 }, {  10, 11 }, {  16, 11 },
-            {  40, 33 }, {  24, 11 }, {  20, 11 }, {  32, 11 },
-            {  80, 33 }, {  18, 11 }, {  15, 11 }, {  64, 33 },
-            { 160, 99 }, {   4,  3 }, {   3,  2 }, {   2,  1 },
-        };
         int num, den, i;
 
         av_reduce(&num, &den, ctx->sample_aspect_ratio.num,
                   ctx->sample_aspect_ratio.den, 65535);
 
-        for (i = 1; i < FF_ARRAY_ELEMS(sar_idc); i++) {
-            if (num == sar_idc[i].num &&
-                den == sar_idc[i].den)
+        for (i = 1; i < FF_ARRAY_ELEMS(ff_h2645_pixel_aspect); i++) {
+            if (num == ff_h2645_pixel_aspect[i].num &&
+                den == ff_h2645_pixel_aspect[i].den)
                 break;
         }
-        if (i == FF_ARRAY_ELEMS(sar_idc)) {
+        if (i == FF_ARRAY_ELEMS(ff_h2645_pixel_aspect)) {
             sps->vui.aspect_ratio_idc = 255;
             sps->vui.sar_width  = num;
             sps->vui.sar_height = den;
@@ -476,12 +470,13 @@ static int h264_metadata_update_fragment(AVBSFContext *bsf, AVPacket *pkt,
     H264MetadataContext *ctx = bsf->priv_data;
     int err, i, has_sps, seek_point;
 
-    // If an AUD is present, it must be the first NAL unit.
-    if (au->nb_units && au->units[0].type == H264_NAL_AUD) {
-        if (ctx->aud == BSF_ELEMENT_REMOVE)
-            ff_cbs_delete_unit(au, 0);
-    } else {
-        if (pkt && ctx->aud == BSF_ELEMENT_INSERT) {
+    if (ctx->aud == BSF_ELEMENT_REMOVE) {
+        for (i = au->nb_units - 1; i >= 0; i--) {
+            if (au->units[i].type == H264_NAL_AUD)
+                ff_cbs_delete_unit(au, i);
+        }
+    } else if (ctx->aud == BSF_ELEMENT_INSERT) {
+        if (pkt) {
             err = h264_metadata_insert_aud(bsf, au);
             if (err < 0)
                 return err;
