@@ -35,7 +35,6 @@
  */
 
 #include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 
 #include "libavutil/intreadwrite.h"
@@ -47,7 +46,6 @@
 #include "decode.h"
 #include "get_bits.h"
 #include "hpeldsp.h"
-#include "internal.h"
 
 #define PALETTE_COUNT 256
 
@@ -928,7 +926,7 @@ static void ipvideo_format_06_firstpass(IpvideoContext *s, AVFrame *frame, int16
         }
     } else {
         /* Don't try to copy second_last_frame data on the first frames */
-        if (s->avctx->frame_number > 2)
+        if (s->avctx->frame_num > 2)
             copy_from(s, s->second_last_frame, frame, 0, 0);
     }
 }
@@ -1087,7 +1085,7 @@ static void ipvideo_decode_format_10_opcodes(IpvideoContext *s, AVFrame *frame)
                 copy_from(s, s->cur_decode_frame, frame, 0, 0);
             } else {
                 /* Don't try to copy last_frame data on the first frame */
-                if (s->avctx->frame_number)
+                if (s->avctx->frame_num)
                     copy_from(s, s->last_frame, frame, 0, 0);
             }
             skip *= 2;
@@ -1146,8 +1144,8 @@ static void ipvideo_decode_format_11_opcodes(IpvideoContext *s, AVFrame *frame)
                 ret = ipvideo_decode_block16[opcode](s, frame);
             }
             if (ret != 0) {
-                av_log(s->avctx, AV_LOG_ERROR, "decode problem on frame %d, @ block (%d, %d)\n",
-                       s->avctx->frame_number, x, y);
+                av_log(s->avctx, AV_LOG_ERROR, "decode problem on frame %"PRId64", @ block (%d, %d)\n",
+                       s->avctx->frame_num, x, y);
                 return;
             }
         }
@@ -1317,7 +1315,14 @@ static int ipvideo_decode_frame(AVCodecContext *avctx, AVFrame *frame,
         return ret;
 
     if (!s->is_16bpp) {
-        frame->palette_has_changed = ff_copy_palette(s->pal, avpkt, avctx);
+#if FF_API_PALETTE_HAS_CHANGED
+FF_DISABLE_DEPRECATION_WARNINGS
+        frame->palette_has_changed =
+#endif
+        ff_copy_palette(s->pal, avpkt, avctx);
+#if FF_API_PALETTE_HAS_CHANGED
+FF_ENABLE_DEPRECATION_WARNINGS
+#endif
     }
 
     switch (frame_format) {
@@ -1335,9 +1340,8 @@ static int ipvideo_decode_frame(AVCodecContext *avctx, AVFrame *frame,
     *got_frame = send_buffer;
 
     /* shuffle frames */
-    av_frame_unref(s->second_last_frame);
     FFSWAP(AVFrame*, s->second_last_frame, s->last_frame);
-    if ((ret = av_frame_ref(s->last_frame, frame)) < 0)
+    if ((ret = av_frame_replace(s->last_frame, frame)) < 0)
         return ret;
 
     /* report that the buffer was completely consumed */
@@ -1358,7 +1362,7 @@ static av_cold int ipvideo_decode_end(AVCodecContext *avctx)
 
 const FFCodec ff_interplay_video_decoder = {
     .p.name         = "interplayvideo",
-    .p.long_name    = NULL_IF_CONFIG_SMALL("Interplay MVE video"),
+    CODEC_LONG_NAME("Interplay MVE video"),
     .p.type         = AVMEDIA_TYPE_VIDEO,
     .p.id           = AV_CODEC_ID_INTERPLAY_VIDEO,
     .priv_data_size = sizeof(IpvideoContext),
@@ -1366,5 +1370,5 @@ const FFCodec ff_interplay_video_decoder = {
     .close          = ipvideo_decode_end,
     FF_CODEC_DECODE_CB(ipvideo_decode_frame),
     .p.capabilities = AV_CODEC_CAP_DR1 | AV_CODEC_CAP_PARAM_CHANGE,
-    .caps_internal  = FF_CODEC_CAP_INIT_THREADSAFE | FF_CODEC_CAP_INIT_CLEANUP,
+    .caps_internal  = FF_CODEC_CAP_INIT_CLEANUP,
 };
