@@ -31,13 +31,18 @@
 /**
  * Apply Welch window function to audio block
  */
-static void lpc_apply_welch_window_c(const int32_t *data, int len,
+static void lpc_apply_welch_window_c(const int32_t *data, ptrdiff_t len,
                                      double *w_data)
 {
     int i, n2;
     double w;
     double c;
 
+    if (len == 1) {
+        w_data[0] = 0.0;
+        return;
+    }
+
     n2 = (len >> 1);
     c = 2.0 / (len - 1.0);
 
@@ -48,6 +53,7 @@ static void lpc_apply_welch_window_c(const int32_t *data, int len,
             w_data[i] = data[i] * w;
             w_data[len-1-i] = data[len-1-i] * w;
         }
+        w_data[n2] = 0.0;
         return;
     }
 
@@ -65,7 +71,7 @@ static void lpc_apply_welch_window_c(const int32_t *data, int len,
  * Calculate autocorrelation data from audio samples
  * A Welch window function is applied before calculation.
  */
-static void lpc_compute_autocorr_c(const double *data, int len, int lag,
+static void lpc_compute_autocorr_c(const double *data, ptrdiff_t len, int lag,
                                    double *autoc)
 {
     int i, j;
