@@ -22,6 +22,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "libavutil/cpu.h"
 #include "libavutil/opt.h"
 #include "libavutil/pixdesc.h"
 #include "libavutil/qsort.h"
@@ -52,10 +53,17 @@ typedef struct PTable {
     int64_t prob;   ///< number of occurences of this value in input
 } PTable;
 
+typedef struct Slice {
+    unsigned pos;
+    unsigned size;
+    uint8_t *slice;
+    uint8_t *bitslice;
+    PTable counts[256];
+} Slice;
+
 typedef struct MagicYUVContext {
     const AVClass       *class;
     int                  frame_pred;
-    PutBitContext        pb;
     int                  planes;
     uint8_t              format;
     int                  slice_height;
@@ -63,10 +71,9 @@ typedef struct MagicYUVContext {
     int                  correlate;
     int                  hshift[4];
     int                  vshift[4];
-    uint8_t             *slices[4];
-    unsigned             slice_pos[4];
-    unsigned             tables_size;
+    unsigned             bitslice_size;
     uint8_t             *decorrelate_buf[2];
+    Slice               *slices;
     HuffEntry            he[4][256];
     LLVidEncDSPContext   llvidencdsp;
     void (*predict)(struct MagicYUVContext *s, const uint8_t *src, uint8_t *dst,
@@ -150,7 +157,6 @@ static av_cold int magy_encode_init(AVCodecContext *avctx)
 {
     MagicYUVContext *s = avctx->priv_data;
     PutByteContext pb;
-    int i;
 
     switch (avctx->pix_fmt) {
     case AV_PIX_FMT_GBRP:
@@ -190,25 +196,39 @@ static av_cold int magy_encode_init(AVCodecContext *avctx)
         s->format = 0x6b;
         break;
     }
-    if (s->correlate) {
-        s->decorrelate_buf[0] = av_calloc(2U * avctx->height, FFALIGN(avctx->width, 16));
-        if (!s->decorrelate_buf[0])
-            return AVERROR(ENOMEM);
-        s->decorrelate_buf[1] = s->decorrelate_buf[0] + avctx->height * FFALIGN(avctx->width, 16);
-    }
 
     ff_llvidencdsp_init(&s->llvidencdsp);
 
     s->planes = av_pix_fmt_count_planes(avctx->pix_fmt);
 
-    s->nb_slices = 1;
+    s->nb_slices = (avctx->slices <= 0) ? av_cpu_count() : avctx->slices;
+    s->nb_slices = FFMIN(s->nb_slices, avctx->height >> s->vshift[1]);
+    s->nb_slices = FFMAX(1, s->nb_slices);
+    s->slice_height = FFALIGN((avctx->height + s->nb_slices - 1) / s->nb_slices, 1 << s->vshift[1]);
+    s->nb_slices = (avctx->height + s->slice_height - 1) / s->slice_height;
+    s->slices = av_calloc(s->nb_slices * s->planes, sizeof(*s->slices));
+    if (!s->slices)
+        return AVERROR(ENOMEM);
 
-    for (i = 0; i < s->planes; i++) {
-        s->slices[i] = av_malloc(avctx->width * (avctx->height + 2) +
-                                 AV_INPUT_BUFFER_PADDING_SIZE);
-        if (!s->slices[i]) {
-            av_log(avctx, AV_LOG_ERROR, "Cannot allocate temporary buffer.\n");
+    if (s->correlate) {
+        s->decorrelate_buf[0] = av_calloc(2U * (s->nb_slices * s->slice_height), FFALIGN(avctx->width, av_cpu_max_align()));
+        if (!s->decorrelate_buf[0])
             return AVERROR(ENOMEM);
+        s->decorrelate_buf[1] = s->decorrelate_buf[0] + (s->nb_slices * s->slice_height) * FFALIGN(avctx->width, av_cpu_max_align());
+    }
+
+    s->bitslice_size = avctx->width * s->slice_height + 2;
+    for (int n = 0; n < s->nb_slices; n++) {
+        for (int i = 0; i < s->planes; i++) {
+            Slice *sl = &s->slices[n * s->planes + i];
+
+            sl->bitslice = av_malloc(s->bitslice_size + AV_INPUT_BUFFER_PADDING_SIZE);
+            sl->slice = av_malloc(avctx->width * (s->slice_height + 2) +
+                                                     AV_INPUT_BUFFER_PADDING_SIZE);
+            if (!sl->slice || !sl->bitslice) {
+                av_log(avctx, AV_LOG_ERROR, "Cannot allocate temporary buffer.\n");
+                return AVERROR(ENOMEM);
+            }
         }
     }
 
@@ -263,15 +283,12 @@ static void calculate_codes(HuffEntry *he, uint16_t codes_count[33])
     }
 }
 
-static void count_usage(uint8_t *src, int width,
+static void count_usage(const uint8_t *src, int width,
                         int height, PTable *counts)
 {
-    int i, j;
-
-    for (j = 0; j < height; j++) {
-        for (i = 0; i < width; i++) {
+    for (int j = 0; j < height; j++) {
+        for (int i = 0; i < width; i++)
             counts[src[i]].prob++;
-        }
         src += width;
     }
 }
@@ -352,17 +369,40 @@ static void magy_huffman_compute_bits(PTable *prob_table, HuffEntry *distincts,
     }
 }
 
-static int encode_table(AVCodecContext *avctx, uint8_t *dst,
-                        int width, int height,
-                        PutBitContext *pb, HuffEntry *he)
+static int count_plane_slice(AVCodecContext *avctx, int n, int plane)
 {
+    MagicYUVContext *s = avctx->priv_data;
+    Slice *sl = &s->slices[n * s->planes + plane];
+    const uint8_t *dst = sl->slice;
+    PTable *counts = sl->counts;
+    const int slice_height = s->slice_height;
+    const int last_height = FFMIN(slice_height, avctx->height - n * slice_height);
+    const int height = (n < (s->nb_slices - 1)) ? slice_height : last_height;
+
+    memset(counts, 0, sizeof(sl->counts));
+
+    count_usage(dst, AV_CEIL_RSHIFT(avctx->width, s->hshift[plane]),
+                AV_CEIL_RSHIFT(height, s->vshift[plane]), counts);
+
+    return 0;
+}
+
+static int encode_table(AVCodecContext *avctx,
+                        PutBitContext *pb, HuffEntry *he, int plane)
+{
+    MagicYUVContext *s = avctx->priv_data;
     PTable counts[256] = { {0} };
     uint16_t codes_counts[33] = { 0 };
-    int i;
 
-    count_usage(dst, width, height, counts);
+    for (int n = 0; n < s->nb_slices; n++) {
+        Slice *sl = &s->slices[n * s->planes + plane];
+        PTable *slice_counts = sl->counts;
 
-    for (i = 0; i < 256; i++) {
+        for (int i = 0; i < 256; i++)
+            counts[i].prob = slice_counts[i].prob;
+    }
+
+    for (int i = 0; i < 256; i++) {
         counts[i].prob++;
         counts[i].value = i;
     }
@@ -371,7 +411,7 @@ static int encode_table(AVCodecContext *avctx, uint8_t *dst,
 
     calculate_codes(he, codes_counts);
 
-    for (i = 0; i < 256; i++) {
+    for (int i = 0; i < 256; i++) {
         put_bits(pb, 1, 0);
         put_bits(pb, 7, he[i].len);
     }
@@ -379,11 +419,28 @@ static int encode_table(AVCodecContext *avctx, uint8_t *dst,
     return 0;
 }
 
-static int encode_slice(uint8_t *src, uint8_t *dst, int dst_size,
-                        int width, int height, HuffEntry *he, int prediction)
+static int encode_plane_slice_raw(const uint8_t *src, uint8_t *dst, unsigned dst_size,
+                                  int width, int height, int prediction)
 {
+    unsigned count = width * height;
+
+    dst[0] = 1;
+    dst[1] = prediction;
+
+    memcpy(dst + 2, src, count);
+    count += 2;
+    AV_WN32(dst + count, 0);
+    if (count & 3)
+        count += 4 - (count & 3);
+
+    return count;
+}
+
+static int encode_plane_slice(const uint8_t *src, uint8_t *dst, unsigned dst_size,
+                              int width, int height, HuffEntry *he, int prediction)
+{
+    const uint8_t *osrc = src;
     PutBitContext pb;
-    int i, j;
     int count;
 
     init_put_bits(&pb, dst, dst_size);
@@ -391,10 +448,13 @@ static int encode_slice(uint8_t *src, uint8_t *dst, int dst_size,
     put_bits(&pb, 8, 0);
     put_bits(&pb, 8, prediction);
 
-    for (j = 0; j < height; j++) {
-        for (i = 0; i < width; i++) {
+    for (int j = 0; j < height; j++) {
+        for (int i = 0; i < width; i++) {
             const int idx = src[i];
-            put_bits(&pb, he[idx].len, he[idx].code);
+            const int len = he[idx].len;
+            if (put_bits_left(&pb) < len + 32)
+                return encode_plane_slice_raw(osrc, dst, dst_size, width, height, prediction);
+            put_bits(&pb, len, he[idx].code);
         }
 
         src += width;
@@ -410,13 +470,98 @@ static int encode_slice(uint8_t *src, uint8_t *dst, int dst_size,
     return put_bytes_output(&pb);
 }
 
+static int encode_slice(AVCodecContext *avctx, void *tdata,
+                        int n, int threadnr)
+{
+    MagicYUVContext *s = avctx->priv_data;
+    const int slice_height = s->slice_height;
+    const int last_height = FFMIN(slice_height, avctx->height - n * slice_height);
+    const int height = (n < (s->nb_slices - 1)) ? slice_height : last_height;
+
+    for (int i = 0; i < s->planes; i++) {
+        Slice *sl = &s->slices[n * s->planes + i];
+
+        sl->size =
+            encode_plane_slice(sl->slice,
+                               sl->bitslice,
+                               s->bitslice_size,
+                               AV_CEIL_RSHIFT(avctx->width, s->hshift[i]),
+                               AV_CEIL_RSHIFT(height, s->vshift[i]),
+                               s->he[i], s->frame_pred);
+    }
+
+    return 0;
+}
+
+static int predict_slice(AVCodecContext *avctx, void *tdata,
+                         int n, int threadnr)
+{
+    const int aligned_width = FFALIGN(avctx->width, av_cpu_max_align());
+    MagicYUVContext *s = avctx->priv_data;
+    const int slice_height = s->slice_height;
+    const int last_height = FFMIN(slice_height, avctx->height - n * slice_height);
+    const int height = (n < (s->nb_slices - 1)) ? slice_height : last_height;
+    const int width = avctx->width;
+    AVFrame *frame = tdata;
+
+    if (s->correlate) {
+        uint8_t *decorrelated[2] = { s->decorrelate_buf[0] + n * slice_height * aligned_width,
+                                     s->decorrelate_buf[1] + n * slice_height * aligned_width };
+        const int decorrelate_linesize = aligned_width;
+        const uint8_t *const data[4] = { decorrelated[0], frame->data[0] + n * slice_height * frame->linesize[0],
+                                         decorrelated[1], s->planes == 4 ? frame->data[3] + n * slice_height * frame->linesize[3] : NULL };
+        const uint8_t *r, *g, *b;
+        const int linesize[4]  = { decorrelate_linesize, frame->linesize[0],
+                                   decorrelate_linesize, frame->linesize[3] };
+
+        g = frame->data[0] + n * slice_height * frame->linesize[0];
+        b = frame->data[1] + n * slice_height * frame->linesize[1];
+        r = frame->data[2] + n * slice_height * frame->linesize[2];
+
+        for (int i = 0; i < height; i++) {
+            s->llvidencdsp.diff_bytes(decorrelated[0], b, g, width);
+            s->llvidencdsp.diff_bytes(decorrelated[1], r, g, width);
+            g += frame->linesize[0];
+            b += frame->linesize[1];
+            r += frame->linesize[2];
+            decorrelated[0] += decorrelate_linesize;
+            decorrelated[1] += decorrelate_linesize;
+        }
+
+        for (int i = 0; i < s->planes; i++) {
+            Slice *sl = &s->slices[n * s->planes + i];
+
+            s->predict(s, data[i], sl->slice, linesize[i],
+                       frame->width, height);
+        }
+    } else {
+        for (int i = 0; i < s->planes; i++) {
+            Slice *sl = &s->slices[n * s->planes + i];
+
+            s->predict(s, frame->data[i] + n * (slice_height >> s->vshift[i]) * frame->linesize[i],
+                       sl->slice,
+                       frame->linesize[i],
+                       AV_CEIL_RSHIFT(frame->width, s->hshift[i]),
+                       AV_CEIL_RSHIFT(height, s->vshift[i]));
+        }
+    }
+
+    for (int p = 0; p < s->planes; p++)
+        count_plane_slice(avctx, n, p);
+
+    return 0;
+}
+
 static int magy_encode_frame(AVCodecContext *avctx, AVPacket *pkt,
                              const AVFrame *frame, int *got_packet)
 {
     MagicYUVContext *s = avctx->priv_data;
-    PutByteContext pb;
     const int width = avctx->width, height = avctx->height;
-    int pos, slice, i, j, ret = 0;
+    const int slice_height = s->slice_height;
+    unsigned tables_size;
+    PutBitContext pbit;
+    PutByteContext pb;
+    int pos, ret = 0;
 
     ret = ff_alloc_packet(avctx, pkt, (256 + 4 * s->nb_slices + width * height) *
                           s->planes + 256);
@@ -439,91 +584,53 @@ static int magy_encode_frame(AVCodecContext *avctx, AVPacket *pkt,
     bytestream2_put_le32(&pb, avctx->width);
     bytestream2_put_le32(&pb, avctx->height);
     bytestream2_put_le32(&pb, avctx->width);
-    bytestream2_put_le32(&pb, avctx->height);
+    bytestream2_put_le32(&pb, slice_height);
     bytestream2_put_le32(&pb, 0);
 
-    for (i = 0; i < s->planes; i++) {
+    for (int i = 0; i < s->planes; i++) {
         bytestream2_put_le32(&pb, 0);
-        for (j = 1; j < s->nb_slices; j++) {
+        for (int j = 1; j < s->nb_slices; j++)
             bytestream2_put_le32(&pb, 0);
-        }
     }
 
     bytestream2_put_byte(&pb, s->planes);
 
-    for (i = 0; i < s->planes; i++) {
-        for (slice = 0; slice < s->nb_slices; slice++) {
-            bytestream2_put_byte(&pb, i);
-        }
+    for (int i = 0; i < s->planes; i++) {
+        for (int n = 0; n < s->nb_slices; n++)
+            bytestream2_put_byte(&pb, n * s->planes + i);
     }
 
-    if (s->correlate) {
-        uint8_t *r, *g, *b, *decorrelated[2] = { s->decorrelate_buf[0],
-                                                 s->decorrelate_buf[1] };
-        const int decorrelate_linesize = FFALIGN(width, 16);
-        const uint8_t *const data[4] = { decorrelated[0], frame->data[0],
-                                         decorrelated[1], frame->data[3] };
-        const int linesize[4]  = { decorrelate_linesize, frame->linesize[0],
-                                   decorrelate_linesize, frame->linesize[3] };
+    avctx->execute2(avctx, predict_slice, (void *)frame, NULL, s->nb_slices);
 
-        g = frame->data[0];
-        b = frame->data[1];
-        r = frame->data[2];
+    init_put_bits(&pbit, pkt->data + bytestream2_tell_p(&pb), bytestream2_get_bytes_left_p(&pb));
 
-        for (i = 0; i < height; i++) {
-            s->llvidencdsp.diff_bytes(decorrelated[0], b, g, width);
-            s->llvidencdsp.diff_bytes(decorrelated[1], r, g, width);
-            g += frame->linesize[0];
-            b += frame->linesize[1];
-            r += frame->linesize[2];
-            decorrelated[0] += decorrelate_linesize;
-            decorrelated[1] += decorrelate_linesize;
+    for (int i = 0; i < s->planes; i++)
+        encode_table(avctx, &pbit, s->he[i], i);
+
+    tables_size = put_bytes_count(&pbit, 1);
+    bytestream2_skip_p(&pb, tables_size);
+
+    avctx->execute2(avctx, encode_slice, NULL, NULL, s->nb_slices);
+
+    for (int n = 0; n < s->nb_slices; n++) {
+        for (int i = 0; i < s->planes; i++) {
+            Slice *sl = &s->slices[n * s->planes + i];
+
+            sl->pos = bytestream2_tell_p(&pb);
+
+            bytestream2_put_buffer(&pb, sl->bitslice, sl->size);
         }
-
-        for (i = 0; i < s->planes; i++) {
-            for (slice = 0; slice < s->nb_slices; slice++) {
-                s->predict(s, data[i], s->slices[i], linesize[i],
-                           frame->width, frame->height);
-            }
-        }
-    } else {
-        for (i = 0; i < s->planes; i++) {
-            for (slice = 0; slice < s->nb_slices; slice++) {
-                s->predict(s, frame->data[i], s->slices[i], frame->linesize[i],
-                           AV_CEIL_RSHIFT(frame->width, s->hshift[i]),
-                           AV_CEIL_RSHIFT(frame->height, s->vshift[i]));
-            }
-        }
-    }
-
-    init_put_bits(&s->pb, pkt->data + bytestream2_tell_p(&pb), bytestream2_get_bytes_left_p(&pb));
-
-    for (i = 0; i < s->planes; i++) {
-        encode_table(avctx, s->slices[i],
-                     AV_CEIL_RSHIFT(frame->width,  s->hshift[i]),
-                     AV_CEIL_RSHIFT(frame->height, s->vshift[i]),
-                     &s->pb, s->he[i]);
-    }
-    s->tables_size = put_bytes_count(&s->pb, 1);
-    bytestream2_skip_p(&pb, s->tables_size);
-
-    for (i = 0; i < s->planes; i++) {
-        unsigned slice_size;
-
-        s->slice_pos[i] = bytestream2_tell_p(&pb);
-        slice_size = encode_slice(s->slices[i], pkt->data + bytestream2_tell_p(&pb),
-                                  bytestream2_get_bytes_left_p(&pb),
-                                  AV_CEIL_RSHIFT(frame->width,  s->hshift[i]),
-                                  AV_CEIL_RSHIFT(frame->height, s->vshift[i]),
-                                  s->he[i], s->frame_pred);
-        bytestream2_skip_p(&pb, slice_size);
     }
 
     pos = bytestream2_tell_p(&pb);
     bytestream2_seek_p(&pb, 32, SEEK_SET);
-    bytestream2_put_le32(&pb, s->slice_pos[0] - 32);
-    for (i = 0; i < s->planes; i++) {
-        bytestream2_put_le32(&pb, s->slice_pos[i] - 32);
+    bytestream2_put_le32(&pb, s->slices[0].pos - 32);
+    for (int i = 0; i < s->planes; i++) {
+        for (int n = 0; n < s->nb_slices; n++) {
+            Slice *sl = &s->slices[n * s->planes + i];
+
+            bytestream2_put_le32(&pb, sl->pos - 32);
+        }
     }
     bytestream2_seek_p(&pb, pos, SEEK_SET);
 
@@ -537,10 +644,14 @@ static int magy_encode_frame(AVCodecContext *avctx, AVPacket *pkt,
 static av_cold int magy_encode_close(AVCodecContext *avctx)
 {
     MagicYUVContext *s = avctx->priv_data;
-    int i;
 
-    for (i = 0; i < s->planes; i++)
-        av_freep(&s->slices[i]);
+    for (int i = 0; i < s->planes * s->nb_slices && s->slices; i++) {
+        Slice *sl = &s->slices[i];
+
+        av_freep(&sl->slice);
+        av_freep(&sl->bitslice);
+    }
+    av_freep(&s->slices);
     av_freep(&s->decorrelate_buf);
 
     return 0;
@@ -565,19 +676,21 @@ static const AVClass magicyuv_class = {
 
 const FFCodec ff_magicyuv_encoder = {
     .p.name           = "magicyuv",
-    .p.long_name      = NULL_IF_CONFIG_SMALL("MagicYUV video"),
+    CODEC_LONG_NAME("MagicYUV video"),
     .p.type           = AVMEDIA_TYPE_VIDEO,
     .p.id             = AV_CODEC_ID_MAGICYUV,
+    .p.capabilities   = AV_CODEC_CAP_DR1 | AV_CODEC_CAP_FRAME_THREADS |
+                        AV_CODEC_CAP_SLICE_THREADS |
+                        AV_CODEC_CAP_ENCODER_REORDERED_OPAQUE,
     .priv_data_size   = sizeof(MagicYUVContext),
     .p.priv_class     = &magicyuv_class,
     .init             = magy_encode_init,
     .close            = magy_encode_close,
     FF_CODEC_ENCODE_CB(magy_encode_frame),
-    .p.capabilities   = AV_CODEC_CAP_FRAME_THREADS,
     .p.pix_fmts       = (const enum AVPixelFormat[]) {
                           AV_PIX_FMT_GBRP, AV_PIX_FMT_GBRAP, AV_PIX_FMT_YUV422P,
                           AV_PIX_FMT_YUV420P, AV_PIX_FMT_YUV444P, AV_PIX_FMT_YUVA444P, AV_PIX_FMT_GRAY8,
                           AV_PIX_FMT_NONE
                       },
-    .caps_internal    = FF_CODEC_CAP_INIT_THREADSAFE | FF_CODEC_CAP_INIT_CLEANUP,
+    .caps_internal    = FF_CODEC_CAP_INIT_CLEANUP,
 };
