@@ -20,8 +20,7 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#include <stdio.h>
-#include <stdlib.h>
+#include <stddef.h>
 #include <string.h>
 
 #include "libavutil/thread.h"
@@ -30,10 +29,10 @@
 #include "bytestream.h"
 #include "codec_internal.h"
 #include "copy_block.h"
+#include "decode.h"
 #include "mathops.h"
 #include "blockdsp.h"
 #include "get_bits.h"
-#include "internal.h"
 #include "aandcttab.h"
 
 #define CBP_VLC_BITS  9
@@ -624,9 +623,8 @@ static int decode_frame(AVCodecContext *avctx, AVFrame *frame,
     get_qtable(s->intraq_tab[0], s->intra_quant, luma_tab);
     get_qtable(s->intraq_tab[1], s->intra_quant, chroma_tab);
 
-    frame->key_frame = s->is_inter == 0;
-
-    if (frame->key_frame) {
+    if (s->is_inter == 0) {
+        frame->flags |= AV_FRAME_FLAG_KEY;
         ret = decode_intra(avctx, gb, frame);
         if (ret < 0)
             return ret;
@@ -639,13 +637,13 @@ static int decode_frame(AVCodecContext *avctx, AVFrame *frame,
             return AVERROR_INVALIDDATA;
         }
 
+        frame->flags &= ~AV_FRAME_FLAG_KEY;
         ret = decode_inter(avctx, gb, frame, s->prev_frame);
         if (ret < 0)
             return ret;
     }
 
-    av_frame_unref(s->prev_frame);
-    if ((ret = av_frame_ref(s->prev_frame, frame)) < 0)
+    if ((ret = av_frame_replace(s->prev_frame, frame)) < 0)
         return ret;
 
     *got_frame = 1;
@@ -659,7 +657,7 @@ static const uint8_t cbp_bits[] = {
 
 static av_cold void init_static_data(void)
 {
-    INIT_VLC_STATIC_FROM_LENGTHS(&cbp_tab, CBP_VLC_BITS, FF_ARRAY_ELEMS(cbp_bits),
+    VLC_INIT_STATIC_FROM_LENGTHS(&cbp_tab, CBP_VLC_BITS, FF_ARRAY_ELEMS(cbp_bits),
                                  cbp_bits, 1, NULL, 0, 0, 0, 0, 1 << CBP_VLC_BITS);
 }
 
@@ -671,7 +669,7 @@ static av_cold int decode_init(AVCodecContext *avctx)
     avctx->pix_fmt = AV_PIX_FMT_YUV420P;
     avctx->color_range = AVCOL_RANGE_JPEG;
 
-    ff_blockdsp_init(&s->bdsp, avctx);
+    ff_blockdsp_init(&s->bdsp);
 
     s->prev_frame = av_frame_alloc();
     if (!s->prev_frame)
@@ -704,7 +702,7 @@ static av_cold int decode_close(AVCodecContext *avctx)
 
 const FFCodec ff_mv30_decoder = {
     .p.name           = "mv30",
-    .p.long_name      = NULL_IF_CONFIG_SMALL("MidiVid 3.0"),
+    CODEC_LONG_NAME("MidiVid 3.0"),
     .p.type           = AVMEDIA_TYPE_VIDEO,
     .p.id             = AV_CODEC_ID_MV30,
     .priv_data_size   = sizeof(MV30Context),
@@ -713,6 +711,5 @@ const FFCodec ff_mv30_decoder = {
     FF_CODEC_DECODE_CB(decode_frame),
     .flush            = decode_flush,
     .p.capabilities   = AV_CODEC_CAP_DR1,
-    .caps_internal    = FF_CODEC_CAP_INIT_THREADSAFE |
-                        FF_CODEC_CAP_INIT_CLEANUP,
+    .caps_internal    = FF_CODEC_CAP_INIT_CLEANUP,
 };
