@@ -20,18 +20,13 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
 #define CACHED_BITSTREAM_READER !ARCH_X86_32
 #include "libavutil/intreadwrite.h"
 
 #include "avcodec.h"
-#include "bytestream.h"
 #include "codec_internal.h"
+#include "decode.h"
 #include "get_bits.h"
-#include "internal.h"
 #include "lossless_videodsp.h"
 #include "zlib_wrapper.h"
 
@@ -88,7 +83,7 @@ static int build_vlc(AVCodecContext *avctx, VLC *vlc)
     uint8_t xlat[256];
     int cur_node, i, j, pos = 0;
 
-    ff_free_vlc(vlc);
+    ff_vlc_free(vlc);
 
     for (i = 0; i < s->nb_symbols; i++) {
         nodes[i].count = s->prob[i];
@@ -145,7 +140,7 @@ static int build_vlc(AVCodecContext *avctx, VLC *vlc)
 
     get_tree_codes(bits, lens, xlat, nodes, cur_node - 1, 0, 0, &pos);
 
-    return ff_init_vlc_sparse(vlc, 12, pos, lens, 2, 2, bits, 4, 4, xlat, 1, 1, 0);
+    return ff_vlc_init_sparse(vlc, 12, pos, lens, 2, 2, bits, 4, 4, xlat, 1, 1, 0);
 }
 
 static int decode_frame(AVCodecContext *avctx, AVFrame *frame,
@@ -164,9 +159,6 @@ static int decode_frame(AVCodecContext *avctx, AVFrame *frame,
     if (size < 1 || size >= avpkt->size)
         return AVERROR_INVALIDDATA;
 
-    if ((ret = ff_get_buffer(avctx, frame, 0)) < 0)
-        return ret;
-
     if (type == MKTAG('L','Z','Y','V')) {
         z_stream *const zstream = &s->zstream.zstream;
         ret = inflateReset(zstream);
@@ -175,6 +167,9 @@ static int decode_frame(AVCodecContext *avctx, AVFrame *frame,
             return AVERROR_EXTERNAL;
         }
 
+        if ((ret = ff_get_buffer(avctx, frame, 0)) < 0)
+            return ret;
+
         zstream->next_in  = avpkt->data + 8;
         zstream->avail_in = avpkt->size - 8;
 
@@ -225,10 +220,16 @@ static int decode_frame(AVCodecContext *avctx, AVFrame *frame,
             }
         }
 
+        if (get_bits_left(gb) < avctx->height * avctx->width)
+            return AVERROR_INVALIDDATA;
+
         ret = build_vlc(avctx, &s->vlc);
         if (ret < 0)
             return ret;
 
+        if ((ret = ff_get_buffer(avctx, frame, 0)) < 0)
+            return ret;
+
         for (int p = 0; p < 3; p++) {
             int width = avctx->width >> (p > 0);
             ptrdiff_t stride = frame->linesize[p];
@@ -273,7 +274,7 @@ static int decode_frame(AVCodecContext *avctx, AVFrame *frame,
     }
 
     frame->pict_type = AV_PICTURE_TYPE_I;
-    frame->key_frame = 1;
+    frame->flags |= AV_FRAME_FLAG_KEY;
     *got_frame = 1;
 
     return avpkt->size;
@@ -295,14 +296,14 @@ static av_cold int decode_close(AVCodecContext *avctx)
     MVHAContext *s = avctx->priv_data;
 
     ff_inflate_end(&s->zstream);
-    ff_free_vlc(&s->vlc);
+    ff_vlc_free(&s->vlc);
 
     return 0;
 }
 
 const FFCodec ff_mvha_decoder = {
     .p.name           = "mvha",
-    .p.long_name      = NULL_IF_CONFIG_SMALL("MidiVid Archive Codec"),
+    CODEC_LONG_NAME("MidiVid Archive Codec"),
     .p.type           = AVMEDIA_TYPE_VIDEO,
     .p.id             = AV_CODEC_ID_MVHA,
     .priv_data_size   = sizeof(MVHAContext),
@@ -310,6 +311,5 @@ const FFCodec ff_mvha_decoder = {
     .close            = decode_close,
     FF_CODEC_DECODE_CB(decode_frame),
     .p.capabilities   = AV_CODEC_CAP_DR1,
-    .caps_internal    = FF_CODEC_CAP_INIT_THREADSAFE |
-                        FF_CODEC_CAP_INIT_CLEANUP,
+    .caps_internal    = FF_CODEC_CAP_INIT_CLEANUP,
 };
