@@ -23,10 +23,11 @@
 #ifndef AVCODEC_OPUSTAB_H
 #define AVCODEC_OPUSTAB_H
 
-#include "libavutil/mem.h"
-
 #include <stdint.h>
 
+#include "libavutil/attributes_internal.h"
+
+FF_VISIBILITY_PUSH_HIDDEN
 extern const uint8_t  ff_celt_band_end[];
 
 extern const uint8_t  ff_opus_default_coupled_streams[];
@@ -156,8 +157,11 @@ extern const uint16_t ff_celt_qn_exp2[];
 extern const float    ff_celt_postfilter_taps[3][3];
 
 extern const float    ff_celt_window2[120];
-extern const float *const ff_celt_window;
+
+extern const float    ff_celt_window_padded[];
+static const float *const ff_celt_window = &ff_celt_window_padded[8];
 
 extern const uint32_t * const ff_celt_pvq_u_row[15];
+FF_VISIBILITY_POP_HIDDEN
 
 #endif /* AVCODEC_OPUSTAB_H */
