@@ -23,7 +23,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "libavutil/imgutils.h"
 #include "libavutil/internal.h"
 #include "libavutil/intreadwrite.h"
 #include "libavutil/mem.h"
@@ -31,7 +30,7 @@
 #include "avcodec.h"
 #include "bytestream.h"
 #include "codec_internal.h"
-#include "internal.h"
+#include "decode.h"
 
 typedef struct ProSumerContext {
     GetByteContext gb;
@@ -196,7 +195,7 @@ static int decode_frame(AVCodecContext *avctx, AVFrame *frame,
     }
 
     frame->pict_type = AV_PICTURE_TYPE_I;
-    frame->key_frame = 1;
+    frame->flags |= AV_FRAME_FLAG_KEY;
     *got_frame = 1;
 
     return avpkt->size;
@@ -367,7 +366,7 @@ static av_cold int decode_close(AVCodecContext *avctx)
 
 const FFCodec ff_prosumer_decoder = {
     .p.name         = "prosumer",
-    .p.long_name    = NULL_IF_CONFIG_SMALL("Brooktree ProSumer Video"),
+    CODEC_LONG_NAME("Brooktree ProSumer Video"),
     .p.type         = AVMEDIA_TYPE_VIDEO,
     .p.id           = AV_CODEC_ID_PROSUMER,
     .priv_data_size = sizeof(ProSumerContext),
@@ -375,6 +374,5 @@ const FFCodec ff_prosumer_decoder = {
     FF_CODEC_DECODE_CB(decode_frame),
     .close          = decode_close,
     .p.capabilities = AV_CODEC_CAP_DR1,
-    .caps_internal  = FF_CODEC_CAP_INIT_THREADSAFE |
-                      FF_CODEC_CAP_INIT_CLEANUP,
+    .caps_internal  = FF_CODEC_CAP_INIT_CLEANUP,
 };
