@@ -24,13 +24,10 @@
  * VB Video decoder
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-
 #include "avcodec.h"
 #include "bytestream.h"
 #include "codec_internal.h"
-#include "internal.h"
+#include "decode.h"
 
 enum VBFlags {
     VB_HAS_GMC     = 0x01,
@@ -233,7 +230,11 @@ static int decode_frame(AVCodecContext *avctx, AVFrame *frame,
     }
 
     memcpy(frame->data[1], c->pal, AVPALETTE_SIZE);
+#if FF_API_PALETTE_HAS_CHANGED
+FF_DISABLE_DEPRECATION_WARNINGS
     frame->palette_has_changed = flags & VB_HAS_PALETTE;
+FF_ENABLE_DEPRECATION_WARNINGS
+#endif
 
     outptr = frame->data[0];
     srcptr = c->frame;
@@ -280,7 +281,7 @@ static av_cold int decode_end(AVCodecContext *avctx)
 
 const FFCodec ff_vb_decoder = {
     .p.name         = "vb",
-    .p.long_name    = NULL_IF_CONFIG_SMALL("Beam Software VB"),
+    CODEC_LONG_NAME("Beam Software VB"),
     .p.type         = AVMEDIA_TYPE_VIDEO,
     .p.id           = AV_CODEC_ID_VB,
     .priv_data_size = sizeof(VBDecContext),
@@ -288,5 +289,5 @@ const FFCodec ff_vb_decoder = {
     .close          = decode_end,
     FF_CODEC_DECODE_CB(decode_frame),
     .p.capabilities = AV_CODEC_CAP_DR1,
-    .caps_internal  = FF_CODEC_CAP_INIT_THREADSAFE | FF_CODEC_CAP_INIT_CLEANUP,
+    .caps_internal  = FF_CODEC_CAP_INIT_CLEANUP,
 };
