@@ -32,6 +32,9 @@ extern "C" {
 
 extern "C" {
 #include "libavformat/avformat.h"
+#include "libavcodec/bytestream.h"
+#include "libavutil/frame.h"
+#include "libavutil/internal.h"
 #include "libavutil/imgutils.h"
 #include "avdevice.h"
 }
@@ -242,19 +245,32 @@ static int decklink_setup_audio(AVFormatContext *avctx, AVStream *st)
         av_log(avctx, AV_LOG_ERROR, "Only one audio stream is supported!\n");
         return -1;
     }
-    if (c->sample_rate != 48000) {
-        av_log(avctx, AV_LOG_ERROR, "Unsupported sample rate!"
-               " Only 48kHz is supported.\n");
-        return -1;
-    }
-    if (c->channels != 2 && c->channels != 8 && c->channels != 16) {
-        av_log(avctx, AV_LOG_ERROR, "Unsupported number of channels!"
-               " Only 2, 8 or 16 channels are supported.\n");
+
+    if (c->codec_id == AV_CODEC_ID_AC3) {
+        /* Regardless of the number of channels in the codec, we're only
+           using 2 SDI audio channels at 48000Hz */
+        ctx->channels = 2;
+    } else if (c->codec_id == AV_CODEC_ID_PCM_S16LE) {
+        if (c->sample_rate != 48000) {
+            av_log(avctx, AV_LOG_ERROR, "Unsupported sample rate!"
+                   " Only 48kHz is supported.\n");
+            return -1;
+        }
+        if (c->ch_layout.nb_channels != 2 && c->ch_layout.nb_channels != 8 && c->ch_layout.nb_channels != 16) {
+            av_log(avctx, AV_LOG_ERROR, "Unsupported number of channels!"
+                   " Only 2, 8 or 16 channels are supported.\n");
+            return -1;
+        }
+        ctx->channels = c->ch_layout.nb_channels;
+    } else {
+        av_log(avctx, AV_LOG_ERROR, "Unsupported codec specified!"
+               " Only PCM_S16LE and AC-3 are supported.\n");
         return -1;
     }
+
     if (ctx->dlo->EnableAudioOutput(bmdAudioSampleRate48kHz,
                                     bmdAudioSampleType16bitInteger,
-                                    c->channels,
+                                    ctx->channels,
                                     bmdAudioOutputStreamTimestamped) != S_OK) {
         av_log(avctx, AV_LOG_ERROR, "Could not enable audio output!\n");
         return -1;
@@ -265,14 +281,90 @@ static int decklink_setup_audio(AVFormatContext *avctx, AVStream *st)
     }
 
     /* The device expects the sample rate to be fixed. */
-    avpriv_set_pts_info(st, 64, 1, c->sample_rate);
-    ctx->channels = c->channels;
+    avpriv_set_pts_info(st, 64, 1, 48000);
 
     ctx->audio = 1;
 
     return 0;
 }
 
+/* Wrap the AC-3 packet into an S337 payload that is in S16LE format which can be easily
+   injected into the PCM stream.  Note: despite the function name, only AC-3 is implemented */
+static int create_s337_payload(AVPacket *pkt, uint8_t **outbuf, int *outsize)
+{
+    /* Note: if the packet size is not divisible by four, we need to make the actual
+       payload larger to ensure it ends on an two channel S16LE boundary */
+    int payload_size = FFALIGN(pkt->size, 4) + 8;
+    uint16_t bitcount = pkt->size * 8;
+    uint8_t *s337_payload;
+    PutByteContext pb;
+
+    /* Sanity check:  According to SMPTE ST 340:2015 Sec 4.1, the AC-3 sync frame will
+       exactly match the 1536 samples of baseband (PCM) audio that it represents.  */
+    if (pkt->size > 1536)
+        return AVERROR(EINVAL);
+
+    /* Encapsulate AC3 syncframe into SMPTE 337 packet */
+    s337_payload = (uint8_t *) av_malloc(payload_size);
+    if (s337_payload == NULL)
+        return AVERROR(ENOMEM);
+    bytestream2_init_writer(&pb, s337_payload, payload_size);
+    bytestream2_put_le16u(&pb, 0xf872); /* Sync word 1 */
+    bytestream2_put_le16u(&pb, 0x4e1f); /* Sync word 1 */
+    bytestream2_put_le16u(&pb, 0x0001); /* Burst Info, including data type (1=ac3) */
+    bytestream2_put_le16u(&pb, bitcount); /* Length code */
+    for (int i = 0; i < (pkt->size - 1); i += 2)
+        bytestream2_put_le16u(&pb, (pkt->data[i] << 8) | pkt->data[i+1]);
+
+    /* Ensure final payload is aligned on 4-byte boundary */
+    if (pkt->size & 1)
+        bytestream2_put_le16u(&pb, pkt->data[pkt->size - 1] << 8);
+    if ((pkt->size & 3) == 1 || (pkt->size & 3) == 2)
+        bytestream2_put_le16u(&pb, 0);
+
+    *outsize = payload_size;
+    *outbuf = s337_payload;
+    return 0;
+}
+
+static int decklink_setup_subtitle(AVFormatContext *avctx, AVStream *st)
+{
+    int ret = -1;
+
+    switch(st->codecpar->codec_id) {
+#if CONFIG_LIBKLVANC
+    case AV_CODEC_ID_EIA_608:
+        /* No special setup required */
+        ret = 0;
+        break;
+#endif
+    default:
+        av_log(avctx, AV_LOG_ERROR, "Unsupported subtitle codec specified\n");
+        break;
+    }
+
+    return ret;
+}
+
+static int decklink_setup_data(AVFormatContext *avctx, AVStream *st)
+{
+    int ret = -1;
+
+    switch(st->codecpar->codec_id) {
+#if CONFIG_LIBKLVANC
+    case AV_CODEC_ID_SMPTE_2038:
+        /* No specific setup required */
+        ret = 0;
+        break;
+#endif
+    default:
+        av_log(avctx, AV_LOG_ERROR, "Unsupported data codec specified\n");
+        break;
+    }
+
+    return ret;
+}
+
 av_cold int ff_decklink_write_trailer(AVFormatContext *avctx)
 {
     struct decklink_cctx *cctx = (struct decklink_cctx *)avctx->priv_data;
@@ -298,7 +390,9 @@ av_cold int ff_decklink_write_trailer(AVFormatContext *avctx)
 #if CONFIG_LIBKLVANC
     klvanc_context_destroy(ctx->vanc_ctx);
 #endif
+    ff_decklink_packet_queue_end(&ctx->vanc_queue);
 
+    ff_ccfifo_uninit(&ctx->cc_fifo);
     av_freep(&cctx->ctx);
 
     return 0;
@@ -312,7 +406,8 @@ static void construct_cc(AVFormatContext *avctx, struct decklink_ctx *ctx,
     uint16_t *cdp_words;
     uint16_t len;
     uint8_t cc_count;
-    int size, ret, i;
+    size_t size;
+    int ret, i;
 
     const uint8_t *data = av_packet_get_side_data(pkt, AV_PKT_DATA_A53_CC, &size);
     if (!data)
@@ -326,7 +421,7 @@ static void construct_cc(AVFormatContext *avctx, struct decklink_ctx *ctx,
 
     ret = klvanc_set_framerate_EIA_708B(cdp, ctx->bmd_tb_num, ctx->bmd_tb_den);
     if (ret) {
-        av_log(avctx, AV_LOG_ERROR, "Invalid framerate specified: %lld/%lld\n",
+        av_log(avctx, AV_LOG_ERROR, "Invalid framerate specified: %" PRId64 "/%" PRId64 "\n",
                ctx->bmd_tb_num, ctx->bmd_tb_den);
         klvanc_destroy_eia708_cdp(cdp);
         return;
@@ -365,8 +460,108 @@ static void construct_cc(AVFormatContext *avctx, struct decklink_ctx *ctx,
     }
 }
 
+/* See SMPTE ST 2016-3:2009 */
+static void construct_afd(AVFormatContext *avctx, struct decklink_ctx *ctx,
+                          AVPacket *pkt, struct klvanc_line_set_s *vanc_lines,
+                          AVStream *st)
+{
+    struct klvanc_packet_afd_s *afd = NULL;
+    uint16_t *afd_words = NULL;
+    uint16_t len;
+    size_t size;
+    int f1_line = 12, f2_line = 0, ret;
+
+    const uint8_t *data = av_packet_get_side_data(pkt, AV_PKT_DATA_AFD, &size);
+    if (!data || size == 0)
+        return;
+
+    ret = klvanc_create_AFD(&afd);
+    if (ret)
+        return;
+
+    ret = klvanc_set_AFD_val(afd, data[0]);
+    if (ret) {
+        av_log(avctx, AV_LOG_ERROR, "Invalid AFD value specified: %d\n",
+               data[0]);
+        klvanc_destroy_AFD(afd);
+        return;
+    }
+
+    /* Compute the AR flag based on the DAR (see ST 2016-1:2009 Sec 9.1).  Note, we treat
+       anything below 1.4 as 4:3 (as opposed to the standard 1.33), because there are lots
+       of streams in the field that aren't *exactly* 4:3 but a tiny bit larger after doing
+       the math... */
+    if (av_cmp_q((AVRational) {st->codecpar->width * st->codecpar->sample_aspect_ratio.num,
+                    st->codecpar->height * st->codecpar->sample_aspect_ratio.den}, (AVRational) {14, 10}) == 1)
+        afd->aspectRatio = ASPECT_16x9;
+    else
+        afd->aspectRatio = ASPECT_4x3;
+
+    ret = klvanc_convert_AFD_to_words(afd, &afd_words, &len);
+    if (ret) {
+        av_log(avctx, AV_LOG_ERROR, "Failed converting AFD packet to words\n");
+        goto out;
+    }
+
+    ret = klvanc_line_insert(ctx->vanc_ctx, vanc_lines, afd_words, len, f1_line, 0);
+    if (ret) {
+        av_log(avctx, AV_LOG_ERROR, "VANC line insertion failed\n");
+        goto out;
+    }
+
+    /* For interlaced video, insert into both fields.  Switching lines for field 2
+       derived from SMPTE RP 168:2009, Sec 6, Table 2. */
+    switch (ctx->bmd_mode) {
+    case bmdModeNTSC:
+    case bmdModeNTSC2398:
+        f2_line = 273 - 10 + f1_line;
+        break;
+    case bmdModePAL:
+        f2_line = 319 - 6 + f1_line;
+        break;
+    case bmdModeHD1080i50:
+    case bmdModeHD1080i5994:
+    case bmdModeHD1080i6000:
+        f2_line = 569 - 7 + f1_line;
+        break;
+    default:
+        f2_line = 0;
+        break;
+    }
+
+    if (f2_line > 0) {
+        ret = klvanc_line_insert(ctx->vanc_ctx, vanc_lines, afd_words, len, f2_line, 0);
+        if (ret) {
+            av_log(avctx, AV_LOG_ERROR, "VANC line insertion failed\n");
+            goto out;
+        }
+    }
+
+out:
+    if (afd)
+        klvanc_destroy_AFD(afd);
+    if (afd_words)
+        free(afd_words);
+}
+
+/* Parse any EIA-608 subtitles sitting on the queue, and write packet side data
+   that will later be handled by construct_cc... */
+static void parse_608subs(AVFormatContext *avctx, struct decklink_ctx *ctx, AVPacket *pkt)
+{
+    size_t cc_size = ff_ccfifo_getoutputsize(&ctx->cc_fifo);
+    uint8_t *cc_data;
+
+    if (!ff_ccfifo_ccdetected(&ctx->cc_fifo))
+        return;
+
+    cc_data = av_packet_new_side_data(pkt, AV_PKT_DATA_A53_CC, cc_size);
+    if (cc_data)
+        ff_ccfifo_injectbytes(&ctx->cc_fifo, cc_data, cc_size);
+}
+
 static int decklink_construct_vanc(AVFormatContext *avctx, struct decklink_ctx *ctx,
-                                   AVPacket *pkt, decklink_frame *frame)
+                                   AVPacket *pkt, decklink_frame *frame,
+                                   AVStream *st)
 {
     struct klvanc_line_set_s vanc_lines = { 0 };
     int ret = 0, i;
@@ -374,7 +569,61 @@ static int decklink_construct_vanc(AVFormatContext *avctx, struct decklink_ctx *
     if (!ctx->supports_vanc)
         return 0;
 
+    parse_608subs(avctx, ctx, pkt);
     construct_cc(avctx, ctx, pkt, &vanc_lines);
+    construct_afd(avctx, ctx, pkt, &vanc_lines, st);
+
+    /* See if there any pending data packets to process */
+    while (ff_decklink_packet_queue_size(&ctx->vanc_queue) > 0) {
+        AVStream *vanc_st;
+        AVPacket vanc_pkt;
+        int64_t pts;
+
+        pts = ff_decklink_packet_queue_peekpts(&ctx->vanc_queue);
+        if (pts > ctx->last_pts) {
+            /* We haven't gotten to the video frame we are supposed to inject
+               the oldest VANC packet into yet, so leave it on the queue... */
+            break;
+        }
+
+        ret = ff_decklink_packet_queue_get(&ctx->vanc_queue, &vanc_pkt, 1);
+        if (vanc_pkt.pts + 1 < ctx->last_pts) {
+            av_log(avctx, AV_LOG_WARNING, "VANC packet too old, throwing away\n");
+            av_packet_unref(&vanc_pkt);
+            continue;
+        }
+
+        vanc_st = avctx->streams[vanc_pkt.stream_index];
+        if (vanc_st->codecpar->codec_id == AV_CODEC_ID_SMPTE_2038) {
+            struct klvanc_smpte2038_anc_data_packet_s *pkt_2038 = NULL;
+
+            klvanc_smpte2038_parse_pes_payload(vanc_pkt.data, vanc_pkt.size, &pkt_2038);
+            if (pkt_2038 == NULL) {
+                av_log(avctx, AV_LOG_ERROR, "failed to decode SMPTE 2038 PES packet");
+                av_packet_unref(&vanc_pkt);
+                continue;
+            }
+            for (int i = 0; i < pkt_2038->lineCount; i++) {
+                struct klvanc_smpte2038_anc_data_line_s *l = &pkt_2038->lines[i];
+                uint16_t *vancWords = NULL;
+                uint16_t vancWordCount;
+
+                if (klvanc_smpte2038_convert_line_to_words(l, &vancWords,
+                                                           &vancWordCount) < 0)
+                    break;
+
+                ret = klvanc_line_insert(ctx->vanc_ctx, &vanc_lines, vancWords,
+                                         vancWordCount, l->line_number, 0);
+                free(vancWords);
+                if (ret != 0) {
+                    av_log(avctx, AV_LOG_ERROR, "VANC line insertion failed\n");
+                    break;
+                }
+            }
+            klvanc_smpte2038_anc_data_packet_free(pkt_2038);
+        }
+        av_packet_unref(&vanc_pkt);
+    }
 
     IDeckLinkVideoFrameAncillary *vanc;
     int result = ctx->dlo->CreateAncillaryData(bmdFormat10BitYUV, &vanc);
@@ -436,9 +685,11 @@ static int decklink_write_video_packet(AVFormatContext *avctx, AVPacket *pkt)
     AVFrame *avframe = NULL, *tmp = (AVFrame *)pkt->data;
     AVPacket *avpacket = NULL;
     decklink_frame *frame;
-    buffercount_type buffered;
+    uint32_t buffered;
     HRESULT hr;
 
+    ctx->last_pts = FFMAX(ctx->last_pts, pkt->pts);
+
     if (st->codecpar->codec_id == AV_CODEC_ID_WRAPPED_AVFRAME) {
         if (tmp->format != AV_PIX_FMT_UYVY422 ||
             tmp->width  != ctx->bmd_width ||
@@ -464,7 +715,7 @@ static int decklink_write_video_packet(AVFormatContext *avctx, AVPacket *pkt)
         frame = new decklink_frame(ctx, avpacket, st->codecpar->codec_id, ctx->bmd_height, ctx->bmd_width);
 
 #if CONFIG_LIBKLVANC
-        if (decklink_construct_vanc(avctx, ctx, pkt, frame))
+        if (decklink_construct_vanc(avctx, ctx, pkt, frame, st))
             av_log(avctx, AV_LOG_ERROR, "Failed to construct VANC\n");
 #endif
     }
@@ -484,6 +735,9 @@ static int decklink_write_video_packet(AVFormatContext *avctx, AVPacket *pkt)
     ctx->frames_buffer_available_spots--;
     pthread_mutex_unlock(&ctx->mutex);
 
+    if (ctx->first_pts == AV_NOPTS_VALUE)
+        ctx->first_pts = pkt->pts;
+
     /* Schedule frame for playback. */
     hr = ctx->dlo->ScheduleVideoFrame((class IDeckLinkVideoFrame *) frame,
                                       pkt->pts * ctx->bmd_tb_num,
@@ -503,14 +757,14 @@ static int decklink_write_video_packet(AVFormatContext *avctx, AVPacket *pkt)
                " Video may misbehave!\n");
 
     /* Preroll video frames. */
-    if (!ctx->playback_started && pkt->pts > ctx->frames_preroll) {
+    if (!ctx->playback_started && pkt->pts > (ctx->first_pts + ctx->frames_preroll)) {
         av_log(avctx, AV_LOG_DEBUG, "Ending audio preroll.\n");
         if (ctx->audio && ctx->dlo->EndAudioPreroll() != S_OK) {
             av_log(avctx, AV_LOG_ERROR, "Could not end audio preroll!\n");
             return AVERROR(EIO);
         }
         av_log(avctx, AV_LOG_DEBUG, "Starting scheduled playback.\n");
-        if (ctx->dlo->StartScheduledPlayback(0, ctx->bmd_tb_den, 1.0) != S_OK) {
+        if (ctx->dlo->StartScheduledPlayback(ctx->first_pts * ctx->bmd_tb_num, ctx->bmd_tb_den, 1.0) != S_OK) {
             av_log(avctx, AV_LOG_ERROR, "Could not start scheduled playback!\n");
             return AVERROR(EIO);
         }
@@ -524,18 +778,58 @@ static int decklink_write_audio_packet(AVFormatContext *avctx, AVPacket *pkt)
 {
     struct decklink_cctx *cctx = (struct decklink_cctx *)avctx->priv_data;
     struct decklink_ctx *ctx = (struct decklink_ctx *)cctx->ctx;
-    int sample_count = pkt->size / (ctx->channels << 1);
-    buffercount_type buffered;
+    AVStream *st = avctx->streams[pkt->stream_index];
+    int sample_count;
+    uint32_t buffered;
+    uint8_t *outbuf = NULL;
+    int ret = 0;
 
     ctx->dlo->GetBufferedAudioSampleFrameCount(&buffered);
     if (pkt->pts > 1 && !buffered)
         av_log(avctx, AV_LOG_WARNING, "There's no buffered audio."
                " Audio will misbehave!\n");
 
-    if (ctx->dlo->ScheduleAudioSamples(pkt->data, sample_count, pkt->pts,
+    if (st->codecpar->codec_id == AV_CODEC_ID_AC3) {
+        /* Encapsulate AC3 syncframe into SMPTE 337 packet */
+        int outbuf_size;
+        ret = create_s337_payload(pkt, &outbuf, &outbuf_size);
+        if (ret < 0)
+            return ret;
+        sample_count = outbuf_size / 4;
+    } else {
+        sample_count = pkt->size / (ctx->channels << 1);
+        outbuf = pkt->data;
+    }
+
+    if (ctx->dlo->ScheduleAudioSamples(outbuf, sample_count, pkt->pts,
                                        bmdAudioSampleRate48kHz, NULL) != S_OK) {
         av_log(avctx, AV_LOG_ERROR, "Could not schedule audio samples.\n");
-        return AVERROR(EIO);
+        ret = AVERROR(EIO);
+    }
+
+    if (st->codecpar->codec_id == AV_CODEC_ID_AC3)
+        av_freep(&outbuf);
+
+    return ret;
+}
+
+static int decklink_write_subtitle_packet(AVFormatContext *avctx, AVPacket *pkt)
+{
+    struct decklink_cctx *cctx = (struct decklink_cctx *)avctx->priv_data;
+    struct decklink_ctx *ctx = (struct decklink_ctx *)cctx->ctx;
+
+    ff_ccfifo_extractbytes(&ctx->cc_fifo, pkt->data, pkt->size);
+
+    return 0;
+}
+
+static int decklink_write_data_packet(AVFormatContext *avctx, AVPacket *pkt)
+{
+    struct decklink_cctx *cctx = (struct decklink_cctx *)avctx->priv_data;
+    struct decklink_ctx *ctx = (struct decklink_ctx *)cctx->ctx;
+
+    if (ff_decklink_packet_queue_put(&ctx->vanc_queue, pkt) < 0) {
+        av_log(avctx, AV_LOG_WARNING, "Failed to queue DATA packet\n");
     }
 
     return 0;
@@ -557,6 +851,9 @@ av_cold int ff_decklink_write_header(AVFormatContext *avctx)
     ctx->list_formats = cctx->list_formats;
     ctx->preroll      = cctx->preroll;
     ctx->duplex_mode  = cctx->duplex_mode;
+    ctx->first_pts    = AV_NOPTS_VALUE;
+    if (cctx->link > 0 && (unsigned int)cctx->link < FF_ARRAY_ELEMS(decklink_link_conf_map))
+        ctx->link = decklink_link_conf_map[cctx->link];
     cctx->ctx = ctx;
 #if CONFIG_LIBKLVANC
     if (klvanc_context_create(&ctx->vanc_ctx) < 0) {
@@ -568,7 +865,6 @@ av_cold int ff_decklink_write_header(AVFormatContext *avctx)
 
     /* List available devices and exit. */
     if (ctx->list_devices) {
-        av_log(avctx, AV_LOG_WARNING, "The -list_devices option is deprecated and will be removed. Please use ffmpeg -sinks decklink instead.\n");
         ff_decklink_list_devices_legacy(avctx, 0, 1);
         return AVERROR_EXIT;
     }
@@ -603,12 +899,35 @@ av_cold int ff_decklink_write_header(AVFormatContext *avctx)
         } else if (c->codec_type == AVMEDIA_TYPE_VIDEO) {
             if (decklink_setup_video(avctx, st))
                 goto error;
+        } else if (c->codec_type == AVMEDIA_TYPE_DATA) {
+            if (decklink_setup_data(avctx, st))
+                goto error;
+        } else if (c->codec_type == AVMEDIA_TYPE_SUBTITLE) {
+            if (decklink_setup_subtitle(avctx, st))
+                goto error;
         } else {
             av_log(avctx, AV_LOG_ERROR, "Unsupported stream type.\n");
             goto error;
         }
     }
 
+    /* Reconfigure the data/subtitle stream clocks to match the video */
+    for (n = 0; n < avctx->nb_streams; n++) {
+        AVStream *st = avctx->streams[n];
+        AVCodecParameters *c = st->codecpar;
+
+        if(c->codec_type == AVMEDIA_TYPE_DATA ||
+           c->codec_type == AVMEDIA_TYPE_SUBTITLE)
+            avpriv_set_pts_info(st, 64, ctx->bmd_tb_num, ctx->bmd_tb_den);
+    }
+    ff_decklink_packet_queue_init(avctx, &ctx->vanc_queue, cctx->vanc_queue_size);
+
+    ret = ff_ccfifo_init(&ctx->cc_fifo, av_make_q(ctx->bmd_tb_den, ctx->bmd_tb_num), avctx);
+    if (ret < 0) {
+        av_log(ctx, AV_LOG_ERROR, "Failure to setup CC FIFO queue\n");
+        goto error;
+    }
+
     return 0;
 
 error:
@@ -618,16 +937,16 @@ error:
 
 int ff_decklink_write_packet(AVFormatContext *avctx, AVPacket *pkt)
 {
-    struct decklink_cctx *cctx = (struct decklink_cctx *)avctx->priv_data;
-    struct decklink_ctx *ctx = (struct decklink_ctx *)cctx->ctx;
     AVStream *st = avctx->streams[pkt->stream_index];
 
-    ctx->last_pts = FFMAX(ctx->last_pts, pkt->pts);
-
     if      (st->codecpar->codec_type == AVMEDIA_TYPE_VIDEO)
         return decklink_write_video_packet(avctx, pkt);
     else if (st->codecpar->codec_type == AVMEDIA_TYPE_AUDIO)
         return decklink_write_audio_packet(avctx, pkt);
+    else if (st->codecpar->codec_type == AVMEDIA_TYPE_DATA)
+        return decklink_write_data_packet(avctx, pkt);
+    else if (st->codecpar->codec_type == AVMEDIA_TYPE_SUBTITLE)
+        return decklink_write_subtitle_packet(avctx, pkt);
 
     return AVERROR(EIO);
 }
