@@ -29,7 +29,6 @@
 #include "libavutil/fifo.h"
 #include "libavutil/opt.h"
 #include "libavutil/time.h"
-#include "libavcodec/avcodec.h"
 #include "libavformat/avformat.h"
 #include "libavformat/internal.h"
 #include "timefilter.h"
@@ -50,8 +49,8 @@ typedef struct JackData {
     jack_port_t **  ports;
     int             nports;
     TimeFilter *    timefilter;
-    AVFifoBuffer *  new_pkts;
-    AVFifoBuffer *  filled_pkts;
+    AVFifo *        new_pkts;
+    AVFifo *        filled_pkts;
     int             pkt_xrun;
     int             jack_xrun;
 } JackData;
@@ -81,13 +80,14 @@ static int process_callback(jack_nframes_t nframes, void *arg)
                                       self->buffer_size);
 
     /* Check if an empty packet is available, and if there's enough space to send it back once filled */
-    if ((av_fifo_size(self->new_pkts) < sizeof(pkt)) || (av_fifo_space(self->filled_pkts) < sizeof(pkt))) {
+    if (!av_fifo_can_read(self->new_pkts) ||
+        !av_fifo_can_write(self->filled_pkts)) {
         self->pkt_xrun = 1;
         return 0;
     }
 
     /* Retrieve empty (but allocated) packet */
-    av_fifo_generic_read(self->new_pkts, &pkt, sizeof(pkt), NULL);
+    av_fifo_read(self->new_pkts, &pkt, 1);
 
     pkt_data  = (float *) pkt.data;
     latency   = 0;
@@ -106,7 +106,7 @@ static int process_callback(jack_nframes_t nframes, void *arg)
     pkt.pts = (cycle_time - (double) latency / (self->nports * self->sample_rate)) * 1000000.0;
 
     /* Send the now filled packet back, and increase packet counter */
-    av_fifo_generic_write(self->filled_pkts, &pkt, sizeof(pkt), NULL);
+    av_fifo_write(self->filled_pkts, &pkt, 1);
     sem_post(&self->packet_count);
 
     return 0;
@@ -134,12 +134,12 @@ static int supply_new_packets(JackData *self, AVFormatContext *context)
     /* Supply the process callback with new empty packets, by filling the new
      * packets FIFO buffer with as many packets as possible. process_callback()
      * can't do this by itself, because it can't allocate memory in realtime. */
-    while (av_fifo_space(self->new_pkts) >= sizeof(pkt)) {
+    while (av_fifo_can_write(self->new_pkts)) {
         if ((test = av_new_packet(&pkt, pkt_size)) < 0) {
             av_log(context, AV_LOG_ERROR, "Could not create packet of size %d\n", pkt_size);
             return test;
         }
-        av_fifo_generic_write(self->new_pkts, &pkt, sizeof(pkt), NULL);
+        av_fifo_write(self->new_pkts, &pkt, 1);
     }
     return 0;
 }
@@ -167,7 +167,7 @@ static int start_jack(AVFormatContext *context)
 
     /* Register JACK ports */
     for (i = 0; i < self->nports; i++) {
-        char str[16];
+        char str[32];
         snprintf(str, sizeof(str), "input_%d", i + 1);
         self->ports[i] = jack_port_register(self->client, str,
                                             JACK_DEFAULT_AUDIO_TYPE,
@@ -193,9 +193,9 @@ static int start_jack(AVFormatContext *context)
     }
 
     /* Create FIFO buffers */
-    self->filled_pkts = av_fifo_alloc_array(FIFO_PACKETS_NUM, sizeof(AVPacket));
+    self->filled_pkts = av_fifo_alloc2(FIFO_PACKETS_NUM, sizeof(AVPacket), 0);
     /* New packets FIFO with one extra packet for safety against underruns */
-    self->new_pkts    = av_fifo_alloc_array((FIFO_PACKETS_NUM + 1), sizeof(AVPacket));
+    self->new_pkts    = av_fifo_alloc2((FIFO_PACKETS_NUM + 1), sizeof(AVPacket), 0);
     if (!self->new_pkts) {
         jack_client_close(self->client);
         return AVERROR(ENOMEM);
@@ -209,14 +209,13 @@ static int start_jack(AVFormatContext *context)
 
 }
 
-static void free_pkt_fifo(AVFifoBuffer **fifo)
+static void free_pkt_fifo(AVFifo **fifop)
 {
+    AVFifo *fifo = *fifop;
     AVPacket pkt;
-    while (av_fifo_size(*fifo)) {
-        av_fifo_generic_read(*fifo, &pkt, sizeof(pkt), NULL);
+    while (av_fifo_read(fifo, &pkt, 1) >= 0)
         av_packet_unref(&pkt);
-    }
-    av_fifo_freep(fifo);
+    av_fifo_freep2(fifop);
 }
 
 static void stop_jack(JackData *self)
@@ -255,7 +254,7 @@ static int audio_read_header(AVFormatContext *context)
     stream->codecpar->codec_id     = AV_CODEC_ID_PCM_F32LE;
 #endif
     stream->codecpar->sample_rate  = self->sample_rate;
-    stream->codecpar->channels     = self->nports;
+    stream->codecpar->ch_layout.nb_channels = self->nports;
 
     avpriv_set_pts_info(stream, 64, 1, 1000000);  /* 64 bits pts in us */
     return 0;
@@ -313,7 +312,7 @@ static int audio_read_packet(AVFormatContext *context, AVPacket *pkt)
     }
 
     /* Retrieve the packet filled with audio data by process_callback() */
-    av_fifo_generic_read(self->filled_pkts, pkt, sizeof(*pkt), NULL);
+    av_fifo_read(self->filled_pkts, pkt, 1);
 
     if ((test = supply_new_packets(self, context)))
         return test;
@@ -342,7 +341,7 @@ static const AVClass jack_indev_class = {
     .category       = AV_CLASS_CATEGORY_DEVICE_AUDIO_INPUT,
 };
 
-AVInputFormat ff_jack_demuxer = {
+const AVInputFormat ff_jack_demuxer = {
     .name           = "jack",
     .long_name      = NULL_IF_CONFIG_SMALL("JACK Audio Connection Kit"),
     .priv_data_size = sizeof(JackData),
