@@ -22,6 +22,8 @@
 
 #include "libavutil/opt.h"
 #include "avfilter.h"
+#include "filters.h"
+#include "formats.h"
 #include "internal.h"
 #include "audio.h"
 #include "ebur128.h"
@@ -85,7 +87,7 @@ typedef struct LoudNormContext {
     int attack_length;
     int release_length;
 
-    int64_t pts;
+    int64_t pts[30];
     enum FrameType frame_type;
     int above_threshold;
     int prev_nb_samples;
@@ -431,10 +433,9 @@ static int filter_frame(AVFilterLink *inlink, AVFrame *in)
         av_frame_copy_props(out, in);
     }
 
-    if (s->pts == AV_NOPTS_VALUE)
-        s->pts = in->pts;
+    out->pts = s->pts[0];
+    memmove(s->pts, &s->pts[1], (FF_ARRAY_ELEMS(s->pts) - 1) * sizeof(s->pts[0]));
 
-    out->pts = s->pts;
     src = (const double *)in->data[0];
     dst = (double *)out->data[0];
     buf = s->buf;
@@ -455,7 +456,7 @@ static int filter_frame(AVFilterLink *inlink, AVFrame *in)
 
         offset    = pow(10., (s->target_i - global) / 20.);
         offset_tp = true_peak * offset;
-        s->offset = offset_tp < s->target_tp ? offset : s->target_tp - true_peak;
+        s->offset = offset_tp < s->target_tp ? offset : s->target_tp / true_peak;
         s->frame_type = LINEAR_MODE;
     }
 
@@ -501,10 +502,7 @@ static int filter_frame(AVFilterLink *inlink, AVFrame *in)
         true_peak_limiter(s, dst, subframe_length, inlink->ch_layout.nb_channels);
         ff_ebur128_add_frames_double(s->r128_out, dst, subframe_length);
 
-        s->pts +=
-        out->nb_samples =
-        inlink->min_samples =
-        inlink->max_samples = subframe_length;
+        out->nb_samples = subframe_length;
 
         s->frame_type = INNER_FRAME;
         break;
@@ -568,7 +566,6 @@ static int filter_frame(AVFilterLink *inlink, AVFrame *in)
         if (s->index >= 30)
             s->index -= 30;
         s->prev_nb_samples = in->nb_samples;
-        s->pts += in->nb_samples;
         break;
 
     case FINAL_FRAME:
@@ -626,25 +623,22 @@ static int filter_frame(AVFilterLink *inlink, AVFrame *in)
 
         dst = (double *)out->data[0];
         ff_ebur128_add_frames_double(s->r128_out, dst, in->nb_samples);
-        s->pts += in->nb_samples;
         break;
     }
 
     if (in != out)
         av_frame_free(&in);
-
     return ff_filter_frame(outlink, out);
 }
 
-static int request_frame(AVFilterLink *outlink)
+static int flush_frame(AVFilterLink *outlink)
 {
-    int ret;
     AVFilterContext *ctx = outlink->src;
     AVFilterLink *inlink = ctx->inputs[0];
     LoudNormContext *s = ctx->priv;
+    int ret = 0;
 
-    ret = ff_request_frame(inlink);
-    if (ret == AVERROR_EOF && s->frame_type == INNER_FRAME) {
+    if (s->frame_type == INNER_FRAME) {
         double *src;
         double *buf;
         int nb_samples, n, c, offset;
@@ -681,16 +675,66 @@ static int request_frame(AVFilterLink *outlink)
     return ret;
 }
 
+static int activate(AVFilterContext *ctx)
+{
+    AVFilterLink *inlink = ctx->inputs[0];
+    AVFilterLink *outlink = ctx->outputs[0];
+    LoudNormContext *s = ctx->priv;
+    AVFrame *in = NULL;
+    int ret = 0, status;
+    int64_t pts;
+
+    FF_FILTER_FORWARD_STATUS_BACK(outlink, inlink);
+
+    if (s->frame_type != LINEAR_MODE) {
+        int nb_samples;
+
+        if (s->frame_type == FIRST_FRAME) {
+            nb_samples = frame_size(inlink->sample_rate, 3000);
+        } else {
+            nb_samples = frame_size(inlink->sample_rate, 100);
+        }
+
+        ret = ff_inlink_consume_samples(inlink, nb_samples, nb_samples, &in);
+    } else {
+        ret = ff_inlink_consume_frame(inlink, &in);
+    }
+
+    if (ret < 0)
+        return ret;
+    if (ret > 0) {
+        if (s->frame_type == FIRST_FRAME) {
+            const int nb_samples = frame_size(inlink->sample_rate, 100);
+
+            for (int i = 0; i < FF_ARRAY_ELEMS(s->pts); i++)
+                s->pts[i] = in->pts + i * nb_samples;
+        } else if (s->frame_type == LINEAR_MODE) {
+            s->pts[0] = in->pts;
+        } else {
+            s->pts[FF_ARRAY_ELEMS(s->pts) - 1] = in->pts;
+        }
+        ret = filter_frame(inlink, in);
+    }
+    if (ret < 0)
+        return ret;
+
+    if (ff_inlink_acknowledge_status(inlink, &status, &pts)) {
+        ff_outlink_set_status(outlink, status, pts);
+        return flush_frame(outlink);
+    }
+
+    FF_FILTER_FORWARD_WANTED(outlink, inlink);
+
+    return FFERROR_NOT_READY;
+}
+
 static int query_formats(AVFilterContext *ctx)
 {
     LoudNormContext *s = ctx->priv;
-    AVFilterFormats *formats;
-    AVFilterLink *inlink = ctx->inputs[0];
-    AVFilterLink *outlink = ctx->outputs[0];
     static const int input_srate[] = {192000, -1};
     static const enum AVSampleFormat sample_fmts[] = {
-        AV_SAMPLE_FMT_DBL,
-        AV_SAMPLE_FMT_NONE
+            AV_SAMPLE_FMT_DBL,
+            AV_SAMPLE_FMT_NONE
     };
     int ret = ff_set_common_all_channel_counts(ctx);
     if (ret < 0)
@@ -700,19 +744,11 @@ static int query_formats(AVFilterContext *ctx)
     if (ret < 0)
         return ret;
 
-    if (s->frame_type != LINEAR_MODE) {
-        formats = ff_make_format_list(input_srate);
-        if (!formats)
-            return AVERROR(ENOMEM);
-        ret = ff_formats_ref(formats, &inlink->outcfg.samplerates);
-        if (ret < 0)
-            return ret;
-        ret = ff_formats_ref(formats, &outlink->incfg.samplerates);
-        if (ret < 0)
-            return ret;
+    if (s->frame_type == LINEAR_MODE) {
+        return ff_set_common_all_samplerates(ctx);
+    } else {
+        return ff_set_common_samplerates_from_list(ctx, input_srate);
     }
-
-    return 0;
 }
 
 static int config_input(AVFilterLink *inlink)
@@ -749,12 +785,6 @@ static int config_input(AVFilterLink *inlink)
 
     init_gaussian_filter(s);
 
-    if (s->frame_type != LINEAR_MODE) {
-        inlink->min_samples =
-        inlink->max_samples = frame_size(inlink->sample_rate, 3000);
-    }
-
-    s->pts = AV_NOPTS_VALUE;
     s->buf_index =
     s->prev_buf_index =
     s->limiter_buf_index = 0;
@@ -894,15 +924,6 @@ static const AVFilterPad avfilter_af_loudnorm_inputs[] = {
         .name         = "default",
         .type         = AVMEDIA_TYPE_AUDIO,
         .config_props = config_input,
-        .filter_frame = filter_frame,
-    },
-};
-
-static const AVFilterPad avfilter_af_loudnorm_outputs[] = {
-    {
-        .name          = "default",
-        .request_frame = request_frame,
-        .type          = AVMEDIA_TYPE_AUDIO,
     },
 };
 
@@ -912,8 +933,9 @@ const AVFilter ff_af_loudnorm = {
     .priv_size     = sizeof(LoudNormContext),
     .priv_class    = &loudnorm_class,
     .init          = init,
+    .activate      = activate,
     .uninit        = uninit,
     FILTER_INPUTS(avfilter_af_loudnorm_inputs),
-    FILTER_OUTPUTS(avfilter_af_loudnorm_outputs),
+    FILTER_OUTPUTS(ff_audio_default_filterpad),
     FILTER_QUERY_FUNC(query_formats),
 };
