@@ -20,9 +20,10 @@
 
 #include "libavutil/opt.h"
 #include "libavutil/time.h"
+#include "audio.h"
 #include "avfilter.h"
-#include "formats.h"
 #include "internal.h"
+#include "video.h"
 
 enum BenchAction {
     ACTION_START,
@@ -100,20 +101,13 @@ static const AVFilterPad bench_inputs[] = {
     },
 };
 
-static const AVFilterPad bench_outputs[] = {
-    {
-        .name = "default",
-        .type = AVMEDIA_TYPE_VIDEO,
-    },
-};
-
 const AVFilter ff_vf_bench = {
     .name          = "bench",
     .description   = NULL_IF_CONFIG_SMALL("Benchmark part of a filtergraph."),
     .priv_size     = sizeof(BenchContext),
     .init          = init,
     FILTER_INPUTS(bench_inputs),
-    FILTER_OUTPUTS(bench_outputs),
+    FILTER_OUTPUTS(ff_video_default_filterpad),
     .priv_class    = &bench_class,
     .flags         = AVFILTER_FLAG_METADATA_ONLY,
 };
@@ -131,20 +125,13 @@ static const AVFilterPad abench_inputs[] = {
     },
 };
 
-static const AVFilterPad abench_outputs[] = {
-    {
-        .name = "default",
-        .type = AVMEDIA_TYPE_AUDIO,
-    },
-};
-
 const AVFilter ff_af_abench = {
     .name          = "abench",
     .description   = NULL_IF_CONFIG_SMALL("Benchmark part of a filtergraph."),
     .priv_size     = sizeof(BenchContext),
     .init          = init,
     FILTER_INPUTS(abench_inputs),
-    FILTER_OUTPUTS(abench_outputs),
+    FILTER_OUTPUTS(ff_audio_default_filterpad),
     .priv_class    = &abench_class,
     .flags         = AVFILTER_FLAG_METADATA_ONLY,
 };
