@@ -475,7 +475,7 @@ int avfilter_graph_segment_parse(AVFilterGraph *graph, const char *graph_str,
 
     graph_str += strspn(graph_str, WHITESPACES);
 
-    ret = parse_sws_flags(&graph_str, &seg->scale_sws_opts, &graph);
+    ret = parse_sws_flags(&graph_str, &seg->scale_sws_opts, graph);
     if (ret < 0)
         goto fail;
 
@@ -532,8 +532,7 @@ int avfilter_graph_segment_create_filters(AVFilterGraphSegment *seg, int flags)
         for (size_t j = 0; j < ch->nb_filters; j++) {
             AVFilterParams *p = ch->filters[j];
             const AVFilter *f = avfilter_get_by_name(p->filter_name);
-            char inst_name[30], *name = p->instance_name ? p->instance_name :
-                                                           inst_name;
+            char name[64];
 
             // skip already processed filters
             if (p->filter || !p->filter_name)
@@ -546,7 +545,9 @@ int avfilter_graph_segment_create_filters(AVFilterGraphSegment *seg, int flags)
             }
 
             if (!p->instance_name)
-                snprintf(inst_name, sizeof(inst_name), "Parsed_%s_%zu", f->name, idx);
+                snprintf(name, sizeof(name), "Parsed_%s_%zu", f->name, idx);
+            else
+                snprintf(name, sizeof(name), "%s@%s", f->name, p->instance_name);
 
             p->filter = avfilter_graph_alloc_filter(seg->graph, f, name);
             if (!p->filter)
@@ -850,6 +851,32 @@ fail:
     return ret;
 }
 
+// print an error message if some options were not found
+static void log_unknown_opt(const AVFilterGraphSegment *seg)
+{
+    for (size_t i = 0; i < seg->nb_chains; i++) {
+        const AVFilterChain *ch = seg->chains[i];
+
+        for (size_t j = 0; j < ch->nb_filters; j++) {
+            const AVFilterParams *p = ch->filters[j];
+            const AVDictionaryEntry *e;
+
+            if (!p->filter)
+                continue;
+
+            e = av_dict_iterate(p->opts, NULL);
+
+            if (e) {
+                av_log(p->filter, AV_LOG_ERROR,
+                       "Could not set non-existent option '%s' to value '%s'\n",
+                       e->key, e->value);
+                return;
+            }
+        }
+    }
+
+}
+
 int avfilter_graph_segment_apply(AVFilterGraphSegment *seg, int flags,
                                  AVFilterInOut **inputs,
                                  AVFilterInOut **outputs)
@@ -867,6 +894,8 @@ int avfilter_graph_segment_apply(AVFilterGraphSegment *seg, int flags,
 
     ret = avfilter_graph_segment_apply_opts(seg, 0);
     if (ret < 0) {
+        if (ret == AVERROR_OPTION_NOT_FOUND)
+            log_unknown_opt(seg);
         av_log(seg->graph, AV_LOG_ERROR, "Error applying filter options\n");
         return ret;
     }
@@ -908,8 +937,11 @@ int avfilter_graph_parse_ptr(AVFilterGraph *graph, const char *filters,
         goto end;
 
     ret = avfilter_graph_segment_apply_opts(seg, 0);
-    if (ret < 0)
+    if (ret < 0) {
+        if (ret == AVERROR_OPTION_NOT_FOUND)
+            log_unknown_opt(seg);
         goto end;
+    }
 
     ret = avfilter_graph_segment_init(seg, 0);
     if (ret < 0)
@@ -987,6 +1019,9 @@ end:
     avfilter_graph_segment_free(&seg);
 
     if (ret < 0) {
+        av_log(graph, AV_LOG_ERROR, "Error processing filtergraph: %s\n",
+               av_err2str(ret));
+
         while (graph->nb_filters)
             avfilter_free(graph->filters[0]);
         av_freep(&graph->filters);
