@@ -32,7 +32,6 @@
 #include "libavcodec/mathops.h" // for mid_pred(), which is a macro so no link dependency
 #include "avfilter.h"
 #include "drawutils.h"
-#include "formats.h"
 #include "internal.h"
 #include "video.h"
 
@@ -223,7 +222,7 @@ static int parse_psfile(AVFilterContext *ctx, const char *fname)
         int k;
         for (k = 0; k < FF_ARRAY_ELEMS(s->cmyk_adjust[0]); k++) {
             READ16(val);
-            s->cmyk_adjust[i][k] = val / 100.;
+            s->cmyk_adjust[i][k] = val / 100.f;
         }
         ret = register_range(s, i);
         if (ret < 0)
@@ -285,32 +284,25 @@ static int config_input(AVFilterLink *inlink)
     return 0;
 }
 
-static int query_formats(AVFilterContext *ctx)
-{
-    static const enum AVPixelFormat pix_fmts[] = {
-        AV_PIX_FMT_RGB24,  AV_PIX_FMT_BGR24,
-        AV_PIX_FMT_RGBA,   AV_PIX_FMT_BGRA,
-        AV_PIX_FMT_ARGB,   AV_PIX_FMT_ABGR,
-        AV_PIX_FMT_0RGB,   AV_PIX_FMT_0BGR,
-        AV_PIX_FMT_RGB0,   AV_PIX_FMT_BGR0,
-        AV_PIX_FMT_RGB48,  AV_PIX_FMT_BGR48,
-        AV_PIX_FMT_RGBA64, AV_PIX_FMT_BGRA64,
-        AV_PIX_FMT_NONE
-    };
-    AVFilterFormats *fmts_list = ff_make_format_list(pix_fmts);
-    if (!fmts_list)
-        return AVERROR(ENOMEM);
-    return ff_set_common_formats(ctx, fmts_list);
-}
+static const enum AVPixelFormat pix_fmts[] = {
+    AV_PIX_FMT_RGB24,  AV_PIX_FMT_BGR24,
+    AV_PIX_FMT_RGBA,   AV_PIX_FMT_BGRA,
+    AV_PIX_FMT_ARGB,   AV_PIX_FMT_ABGR,
+    AV_PIX_FMT_0RGB,   AV_PIX_FMT_0BGR,
+    AV_PIX_FMT_RGB0,   AV_PIX_FMT_BGR0,
+    AV_PIX_FMT_RGB48,  AV_PIX_FMT_BGR48,
+    AV_PIX_FMT_RGBA64, AV_PIX_FMT_BGRA64,
+    AV_PIX_FMT_NONE
+};
 
 static inline int comp_adjust(int scale, float value, float adjust, float k, int correction_method)
 {
     const float min = -value;
-    const float max = 1. - value;
-    float res = (-1. - adjust) * k - adjust;
+    const float max = 1.f - value;
+    float res = (-1.f - adjust) * k - adjust;
     if (correction_method == CORRECTION_METHOD_RELATIVE)
         res *= max;
-    return lrint(av_clipf(res, min, max) * scale);
+    return lrintf(av_clipf(res, min, max) * scale);
 }
 
 #define DECLARE_SELECTIVE_COLOR_FUNC(nbits)                                                             \
@@ -325,27 +317,37 @@ static inline int selective_color_##nbits(AVFilterContext *ctx, ThreadData *td,
     const int width  = in->width;                                                                       \
     const int slice_start = (height *  jobnr   ) / nb_jobs;                                             \
     const int slice_end   = (height * (jobnr+1)) / nb_jobs;                                             \
-    const int dst_linesize = out->linesize[0];                                                          \
-    const int src_linesize =  in->linesize[0];                                                          \
+    const int dst_linesize = out->linesize[0] / ((nbits + 7) / 8);                                      \
+    const int src_linesize =  in->linesize[0] / ((nbits + 7) / 8);                                      \
     const uint8_t roffset = s->rgba_map[R];                                                             \
     const uint8_t goffset = s->rgba_map[G];                                                             \
     const uint8_t boffset = s->rgba_map[B];                                                             \
     const uint8_t aoffset = s->rgba_map[A];                                                             \
+    uint##nbits##_t *dst = (uint##nbits##_t *)out->data[0] + slice_start * dst_linesize;                \
+    const uint##nbits##_t *src = (const uint##nbits##_t *)in->data[0] + slice_start * src_linesize;     \
+    const uint##nbits##_t *src_r = (const uint##nbits##_t *)src + roffset;                              \
+    const uint##nbits##_t *src_g = (const uint##nbits##_t *)src + goffset;                              \
+    const uint##nbits##_t *src_b = (const uint##nbits##_t *)src + boffset;                              \
+    const uint##nbits##_t *src_a = (const uint##nbits##_t *)src + aoffset;                              \
+    uint##nbits##_t *dst_r = (uint##nbits##_t *)dst + roffset;                                          \
+    uint##nbits##_t *dst_g = (uint##nbits##_t *)dst + goffset;                                          \
+    uint##nbits##_t *dst_b = (uint##nbits##_t *)dst + boffset;                                          \
+    uint##nbits##_t *dst_a = (uint##nbits##_t *)dst + aoffset;                                          \
+    const int mid = 1<<(nbits-1);                                                                       \
+    const int max = (1<<nbits)-1;                                                                       \
+    const float scale = 1.f / max;                                                                      \
                                                                                                         \
     for (y = slice_start; y < slice_end; y++) {                                                         \
-        uint##nbits##_t       *dst = (      uint##nbits##_t *)(out->data[0] + y * dst_linesize);        \
-        const uint##nbits##_t *src = (const uint##nbits##_t *)( in->data[0] + y * src_linesize);        \
-                                                                                                        \
         for (x = 0; x < width * s->step; x += s->step) {                                                \
-            const int r = src[x + roffset];                                                             \
-            const int g = src[x + goffset];                                                             \
-            const int b = src[x + boffset];                                                             \
+            const int r = src_r[x];                                                                     \
+            const int g = src_g[x];                                                                     \
+            const int b = src_b[x];                                                                     \
             const int min_color = FFMIN3(r, g, b);                                                      \
             const int max_color = FFMAX3(r, g, b);                                                      \
-            const int is_white   = (r > 1<<(nbits-1) && g > 1<<(nbits-1) && b > 1<<(nbits-1));          \
+            const int is_white   = (r > mid && g > mid && b > mid);                                     \
             const int is_neutral = (r || g || b) &&                                                     \
-                                   (r != (1<<nbits)-1 || g != (1<<nbits)-1 || b != (1<<nbits)-1);       \
-            const int is_black   = (r < 1<<(nbits-1) && g < 1<<(nbits-1) && b < 1<<(nbits-1));          \
+                                   (r != max || g != max || b != max);                                  \
+            const int is_black   = (r < mid && g < mid && b < mid);                                     \
             const uint32_t range_flag = (r == max_color) << RANGE_REDS                                  \
                                       | (r == min_color) << RANGE_CYANS                                 \
                                       | (g == max_color) << RANGE_GREENS                                \
@@ -356,9 +358,9 @@ static inline int selective_color_##nbits(AVFilterContext *ctx, ThreadData *td,
                                       | is_neutral       << RANGE_NEUTRALS                              \
                                       | is_black         << RANGE_BLACKS;                               \
                                                                                                         \
-            const float rnorm = r * (1.f / ((1<<nbits)-1));                                             \
-            const float gnorm = g * (1.f / ((1<<nbits)-1));                                             \
-            const float bnorm = b * (1.f / ((1<<nbits)-1));                                             \
+            const float rnorm = r * scale;                                                              \
+            const float gnorm = g * scale;                                                              \
+            const float bnorm = b * scale;                                                              \
             int adjust_r = 0, adjust_g = 0, adjust_b = 0;                                               \
                                                                                                         \
             for (i = 0; i < s->nb_process_ranges; i++) {                                                \
@@ -382,13 +384,23 @@ static inline int selective_color_##nbits(AVFilterContext *ctx, ThreadData *td,
             }                                                                                           \
                                                                                                         \
             if (!direct || adjust_r || adjust_g || adjust_b) {                                          \
-                dst[x + roffset] = av_clip_uint##nbits(r + adjust_r);                                   \
-                dst[x + goffset] = av_clip_uint##nbits(g + adjust_g);                                   \
-                dst[x + boffset] = av_clip_uint##nbits(b + adjust_b);                                   \
+                dst_r[x] = av_clip_uint##nbits(r + adjust_r);                                           \
+                dst_g[x] = av_clip_uint##nbits(g + adjust_g);                                           \
+                dst_b[x] = av_clip_uint##nbits(b + adjust_b);                                           \
                 if (!direct && s->step == 4)                                                            \
-                    dst[x + aoffset] = src[x + aoffset];                                                \
+                    dst_a[x] = src_a[x];                                                                \
             }                                                                                           \
         }                                                                                               \
+                                                                                                        \
+        src_r += src_linesize;                                                                          \
+        src_g += src_linesize;                                                                          \
+        src_b += src_linesize;                                                                          \
+        src_a += src_linesize;                                                                          \
+                                                                                                        \
+        dst_r += dst_linesize;                                                                          \
+        dst_g += dst_linesize;                                                                          \
+        dst_b += dst_linesize;                                                                          \
+        dst_a += dst_linesize;                                                                          \
     }                                                                                                   \
     return 0;                                                                                           \
 }
@@ -444,8 +456,8 @@ static int filter_frame(AVFilterLink *inlink, AVFrame *in)
 
     td.in = in;
     td.out = out;
-    ctx->internal->execute(ctx, funcs[s->is_16bit][direct][s->correction_method],
-                           &td, NULL, FFMIN(inlink->h, ff_filter_get_nb_threads(ctx)));
+    ff_filter_execute(ctx, funcs[s->is_16bit][direct][s->correction_method],
+                      &td, NULL, FFMIN(inlink->h, ff_filter_get_nb_threads(ctx)));
 
     if (!direct)
         av_frame_free(&in);
@@ -459,24 +471,15 @@ static const AVFilterPad selectivecolor_inputs[] = {
         .filter_frame = filter_frame,
         .config_props = config_input,
     },
-    { NULL }
 };
 
-static const AVFilterPad selectivecolor_outputs[] = {
-    {
-        .name = "default",
-        .type = AVMEDIA_TYPE_VIDEO,
-    },
-    { NULL }
-};
-
-AVFilter ff_vf_selectivecolor = {
+const AVFilter ff_vf_selectivecolor = {
     .name          = "selectivecolor",
     .description   = NULL_IF_CONFIG_SMALL("Apply CMYK adjustments to specific color ranges."),
     .priv_size     = sizeof(SelectiveColorContext),
-    .query_formats = query_formats,
-    .inputs        = selectivecolor_inputs,
-    .outputs       = selectivecolor_outputs,
+    FILTER_INPUTS(selectivecolor_inputs),
+    FILTER_OUTPUTS(ff_video_default_filterpad),
+    FILTER_PIXFMTS_ARRAY(pix_fmts),
     .priv_class    = &selectivecolor_class,
     .flags         = AVFILTER_FLAG_SUPPORT_TIMELINE_GENERIC | AVFILTER_FLAG_SLICE_THREADS,
 };
