@@ -37,7 +37,6 @@ typedef struct FITSContext {
     const AVClass *class;
     AVRational framerate;
     int first_image;
-    int64_t pts;
 } FITSContext;
 
 static int fits_probe(const AVProbeData *p)
@@ -61,7 +60,6 @@ static int fits_read_header(AVFormatContext *s)
     st->codecpar->codec_id = AV_CODEC_ID_FITS;
 
     avpriv_set_pts_info(st, 64, fits->framerate.den, fits->framerate.num);
-    fits->pts = 0;
     fits->first_image = 1;
     return 0;
 }
@@ -196,13 +194,11 @@ static int fits_read_packet(AVFormatContext *s, AVPacket *pkt)
     pkt->size = avbuf.len - 80;
     av_freep(&buf);
     ret = avio_read(s->pb, pkt->data + pkt->size, size);
-    if (ret < 0) {
+    if (ret < 0)
         return ret;
-    }
 
     pkt->size += ret;
-    pkt->pts = fits->pts;
-    fits->pts++;
+    pkt->duration = 1;
 
     return 0;
 
@@ -221,9 +217,10 @@ static const AVClass fits_demuxer_class = {
     .item_name  = av_default_item_name,
     .option     = fits_options,
     .version    = LIBAVUTIL_VERSION_INT,
+    .category   = AV_CLASS_CATEGORY_DEMUXER,
 };
 
-AVInputFormat ff_fits_demuxer = {
+const AVInputFormat ff_fits_demuxer = {
     .name           = "fits",
     .long_name      = NULL_IF_CONFIG_SMALL("Flexible Image Transport System"),
     .priv_data_size = sizeof(FITSContext),
@@ -231,5 +228,5 @@ AVInputFormat ff_fits_demuxer = {
     .read_header    = fits_read_header,
     .read_packet    = fits_read_packet,
     .priv_class     = &fits_demuxer_class,
-    .raw_codec_id   = AV_CODEC_ID_FITS,
+    .flags          = AVFMT_NOTIMESTAMPS,
 };
